#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
---
module: ce_aaa_server_host
short_description: Manages AAA server host configuration on HUAWEI CloudEngine switches.
description:
    - Manages AAA server host configuration on HUAWEI CloudEngine switches.
author:
    - wangdezhuang (@QijunPan)
notes:
  - This module requires the netconf system service be enabled on the remote device being managed.
  - Recommended connection is C(netconf).
  - This module also works with C(local) connections for legacy playbooks.
options:
    state:
        description:
            - Specify desired state of the resource.
        default: present
        choices: ['present', 'absent']
    local_user_name:
        description:
            - Name of a local user.
              The value is a string of 1 to 253 characters.
    local_password:
        description:
            - Login password of a user. The password can contain letters, numbers, and special characters.
              The value is a string of 1 to 255 characters.
    local_service_type:
        description:
            - The type of local user login through, such as ftp ssh snmp telnet.
    local_ftp_dir:
        description:
            - FTP user directory.
              The value is a string of 1 to 255 characters.
    local_user_level:
        description:
            - Login level of a local user.
              The value is an integer ranging from 0 to 15.
    local_user_group:
        description:
            - Name of the user group where the user belongs. The user inherits all the rights of the user group.
              The value is a string of 1 to 32 characters.
    radius_group_name:
        description:
            - RADIUS server group's name.
              The value is a string of 1 to 32 case-insensitive characters.
    radius_server_type:
        description:
            - Type of Radius Server.
        choices: ['Authentication', 'Accounting']
    radius_server_ip:
        description:
            - IPv4 address of configured server.
              The value is a string of 0 to 255 characters, in dotted decimal notation.
    radius_server_ipv6:
        description:
            - IPv6 address of configured server.
              The total length is 128 bits.
    radius_server_port:
        description:
            - Configured server port for a particular server.
              The value is an integer ranging from 1 to 65535.
    radius_server_mode:
        description:
            - Configured primary or secondary server for a particular server.
        choices: ['Secondary-server', 'Primary-server']
    radius_vpn_name:
        description:
            - Set VPN instance.
              The value is a string of 1 to 31 case-sensitive characters.
    radius_server_name:
        description:
            - Hostname of configured server.
              The value is a string of 0 to 255 case-sensitive characters.
    hwtacacs_template:
        description:
            - Name of a HWTACACS template.
              The value is a string of 1 to 32 case-insensitive characters.
    hwtacacs_server_ip:
        description:
            - Server IPv4 address. Must be a valid unicast IP address.
              The value is a string of 0 to 255 characters, in dotted decimal notation.
    hwtacacs_server_ipv6:
        description:
            - Server IPv6 address. Must be a valid unicast IP address.
              The total length is 128 bits.
    hwtacacs_server_type:
        description:
            - Hwtacacs server type.
        choices: ['Authentication', 'Authorization', 'Accounting', 'Common']
    hwtacacs_is_secondary_server:
        description:
            - Whether the server is secondary.
        type: bool
        default: 'no'
    hwtacacs_vpn_name:
        description:
            - VPN instance name.
    hwtacacs_is_public_net:
        description:
            - Set the public-net.
        type: bool
        default: 'no'
    hwtacacs_server_host_name:
        description:
            - Hwtacacs server host name.
'''

EXAMPLES = '''

- name: AAA server host test
  hosts: cloudengine
  connection: local
  gather_facts: no
  vars:
    cli:
      host: "{{ inventory_hostname }}"
      port: "{{ ansible_ssh_port }}"
      username: "{{ username }}"
      password: "{{ password }}"
      transport: cli

  tasks:

  - name: "Config local user when use local scheme"
    community.network.ce_aaa_server_host:
      state: present
      local_user_name: user1
      local_password: 123456
      provider: "{{ cli }}"

  - name: "Undo local user when use local scheme"
    community.network.ce_aaa_server_host:
      state: absent
      local_user_name: user1
      local_password: 123456
      provider: "{{ cli }}"

  - name: "Config radius server ip"
    community.network.ce_aaa_server_host:
      state: present
      radius_group_name: group1
      radius_server_type: Authentication
      radius_server_ip: 10.1.10.1
      radius_server_port: 2000
      radius_server_mode: Primary-server
      radius_vpn_name: _public_
      provider: "{{ cli }}"

  - name: "Undo radius server ip"
    community.network.ce_aaa_server_host:
      state: absent
      radius_group_name: group1
      radius_server_type: Authentication
      radius_server_ip: 10.1.10.1
      radius_server_port: 2000
      radius_server_mode: Primary-server
      radius_vpn_name: _public_
      provider: "{{ cli }}"

  - name: "Config hwtacacs server ip"
    community.network.ce_aaa_server_host:
      state: present
      hwtacacs_template: template
      hwtacacs_server_ip: 10.10.10.10
      hwtacacs_server_type: Authorization
      hwtacacs_vpn_name: _public_
      provider: "{{ cli }}"

  - name: "Undo hwtacacs server ip"
    community.network.ce_aaa_server_host:
      state: absent
      hwtacacs_template: template
      hwtacacs_server_ip: 10.10.10.10
      hwtacacs_server_type: Authorization
      hwtacacs_vpn_name: _public_
      provider: "{{ cli }}"
'''

RETURN = '''
changed:
    description: check to see if a change was made on the device
    returned: always
    type: bool
    sample: true
proposed:
    description: k/v pairs of parameters passed into module
    returned: always
    type: dict
    sample: {"hwtacacs_is_public_net": "false",
             "hwtacacs_is_secondary_server": "false",
             "hwtacacs_server_ip": "10.135.182.157",
             "hwtacacs_server_type": "Authorization",
             "hwtacacs_template": "wdz",
             "hwtacacs_vpn_name": "_public_",
             "local_password": "******",
             "state": "present"}
existing:
    description: k/v pairs of existing aaa server host
    returned: always
    type: dict
    sample: {"radius server ipv4": []}
end_state:
    description: k/v pairs of aaa params after module execution
    returned: always
    type: dict
    sample: {"radius server ipv4": [
             [
                "10.1.10.1",
                "Authentication",
                "2000",
                "Primary-server",
                "_public_"
              ]
             ]}
updates:
    description: command sent to the device
    returned: always
    type: list
    sample: ["hwtacacs server template test",
             "hwtacacs server authorization 10.135.182.157 vpn-instance test_vpn public-net"]
'''

from xml.etree import ElementTree
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.network.plugins.module_utils.network.cloudengine.ce import get_nc_config, set_nc_config, ce_argument_spec, check_ip_addr

SUCCESS = """success"""
FAILED = """failed"""

INVALID_USER_NAME_CHAR = [' ', '/', '\\',
                          ':', '*', '?', '"', '\'', '<', '>', '%']

# get local user name
CE_GET_LOCAL_USER_INFO_HEADER = """
    <filter type="subtree">
      <aaa xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <lam>
          <users>
            <user>
              <userName></userName>
              <password></password>
"""
CE_GET_LOCAL_USER_INFO_TAIL = """
            </user>
          </users>
        </lam>
      </aaa>
    </filter>
"""

# merge local user name
CE_MERGE_LOCAL_USER_INFO_HEADER = """
    <config>
      <aaa xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <lam>
          <users>
            <user operation="merge">
              <userName>%s</userName>
"""
CE_MERGE_LOCAL_USER_INFO_TAIL = """
            </user>
          </users>
        </lam>
      </aaa>
    </config>
"""

# delete local user name
CE_DELETE_LOCAL_USER_INFO_HEADER = """
    <config>
      <aaa xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <lam>
          <users>
            <user operation="delete">
              <userName>%s</userName>
"""
CE_DELETE_LOCAL_USER_INFO_TAIL = """
            </user>
          </users>
        </lam>
      </aaa>
    </config>
"""

# get radius server config ipv4
CE_GET_RADIUS_SERVER_CFG_IPV4 = """
    <filter type="subtree">
      <radius xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <rdsTemplates>
          <rdsTemplate>
            <groupName>%s</groupName>
            <rdsServerIPV4s>
              <rdsServerIPV4>
                <serverType></serverType>
                <serverIPAddress></serverIPAddress>
                <serverPort></serverPort>
                <serverMode></serverMode>
                <vpnName></vpnName>
              </rdsServerIPV4>
            </rdsServerIPV4s>
          </rdsTemplate>
        </rdsTemplates>
      </radius>
    </filter>
"""

# merge radius server config ipv4
CE_MERGE_RADIUS_SERVER_CFG_IPV4 = """
    <config>
      <radius xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <rdsTemplates>
          <rdsTemplate>
            <groupName>%s</groupName>
            <rdsServerIPV4s>
              <rdsServerIPV4 operation="merge">
                <serverType>%s</serverType>
                <serverIPAddress>%s</serverIPAddress>
                <serverPort>%s</serverPort>
                <serverMode>%s</serverMode>
                <vpnName>%s</vpnName>
              </rdsServerIPV4>
            </rdsServerIPV4s>
          </rdsTemplate>
        </rdsTemplates>
      </radius>
    </config>
"""

# delete radius server config ipv4
CE_DELETE_RADIUS_SERVER_CFG_IPV4 = """
    <config>
      <radius xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <rdsTemplates>
          <rdsTemplate>
            <groupName>%s</groupName>
            <rdsServerIPV4s>
              <rdsServerIPV4 operation="delete">
                <serverType>%s</serverType>
                <serverIPAddress>%s</serverIPAddress>
                <serverPort>%s</serverPort>
                <serverMode>%s</serverMode>
                <vpnName>%s</vpnName>
              </rdsServerIPV4>
            </rdsServerIPV4s>
          </rdsTemplate>
        </rdsTemplates>
      </radius>
    </config>
"""

# get radius server config ipv6
CE_GET_RADIUS_SERVER_CFG_IPV6 = """
    <filter type="subtree">
      <radius xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <rdsTemplates>
          <rdsTemplate>
            <groupName>%s</groupName>
            <rdsServerIPV6s>
              <rdsServerIPV6>
                <serverType></serverType>
                <serverIPAddress></serverIPAddress>
                <serverPort></serverPort>
                <serverMode></serverMode>
              </rdsServerIPV6>
            </rdsServerIPV6s>
          </rdsTemplate>
        </rdsTemplates>
      </radius>
    </filter>
"""

# merge radius server config ipv6
CE_MERGE_RADIUS_SERVER_CFG_IPV6 = """
    <config>
      <radius xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <rdsTemplates>
          <rdsTemplate>
            <groupName>%s</groupName>
            <rdsServerIPV6s>
              <rdsServerIPV6 operation="merge">
                <serverType>%s</serverType>
                <serverIPAddress>%s</serverIPAddress>
                <serverPort>%s</serverPort>
                <serverMode>%s</serverMode>
              </rdsServerIPV6>
            </rdsServerIPV6s>
          </rdsTemplate>
        </rdsTemplates>
      </radius>
    </config>
"""

# delete radius server config ipv6
CE_DELETE_RADIUS_SERVER_CFG_IPV6 = """
    <config>
      <radius xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <rdsTemplates>
          <rdsTemplate>
            <groupName>%s</groupName>
            <rdsServerIPV6s>
              <rdsServerIPV6 operation="delete">
                <serverType>%s</serverType>
                <serverIPAddress>%s</serverIPAddress>
                <serverPort>%s</serverPort>
                <serverMode>%s</serverMode>
              </rdsServerIPV6>
            </rdsServerIPV6s>
          </rdsTemplate>
        </rdsTemplates>
      </radius>
    </config>
"""

# get radius server name
CE_GET_RADIUS_SERVER_NAME = """
    <filter type="subtree">
      <radius xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <rdsTemplates>
          <rdsTemplate>
            <groupName>%s</groupName>
            <rdsServerNames>
              <rdsServerName>
                <serverType></serverType>
                <serverName></serverName>
                <serverPort></serverPort>
                <serverMode></serverMode>
                <vpnName></vpnName>
              </rdsServerName>
            </rdsServerNames>
          </rdsTemplate>
        </rdsTemplates>
      </radius>
    </filter>
"""

# merge radius server name
CE_MERGE_RADIUS_SERVER_NAME = """
    <config>
      <radius xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <rdsTemplates>
          <rdsTemplate>
            <groupName>%s</groupName>
            <rdsServerNames>
              <rdsServerName operation="merge">
                <serverType>%s</serverType>
                <serverName>%s</serverName>
                <serverPort>%s</serverPort>
                <serverMode>%s</serverMode>
                <vpnName>%s</vpnName>
              </rdsServerName>
            </rdsServerNames>
          </rdsTemplate>
        </rdsTemplates>
      </radius>
    </config>
"""

# delete radius server name
CE_DELETE_RADIUS_SERVER_NAME = """
    <config>
      <radius xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <rdsTemplates>
          <rdsTemplate>
            <groupName>%s</groupName>
            <rdsServerNames>
              <rdsServerName operation="delete">
                <serverType>%s</serverType>
                <serverName>%s</serverName>
                <serverPort>%s</serverPort>
                <serverMode>%s</serverMode>
                <vpnName>%s</vpnName>
              </rdsServerName>
            </rdsServerNames>
          </rdsTemplate>
        </rdsTemplates>
      </radius>
    </config>
"""

# get hwtacacs server config ipv4
CE_GET_HWTACACS_SERVER_CFG_IPV4 = """
    <filter type="subtree">
      <hwtacacs xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <hwTacTempCfgs>
          <hwTacTempCfg>
            <templateName>%s</templateName>
            <hwTacSrvCfgs>
              <hwTacSrvCfg>
                <serverIpAddress></serverIpAddress>
                <serverType></serverType>
                <isSecondaryServer></isSecondaryServer>
                <vpnName></vpnName>
                <isPublicNet></isPublicNet>
              </hwTacSrvCfg>
            </hwTacSrvCfgs>
          </hwTacTempCfg>
        </hwTacTempCfgs>
      </hwtacacs>
    </filter>
"""

# merge hwtacacs server config ipv4
CE_MERGE_HWTACACS_SERVER_CFG_IPV4 = """
    <config>
      <hwtacacs xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <hwTacTempCfgs>
          <hwTacTempCfg>
            <templateName>%s</templateName>
            <hwTacSrvCfgs>
              <hwTacSrvCfg operation="merge">
                <serverIpAddress>%s</serverIpAddress>
                <serverType>%s</serverType>
                <isSecondaryServer>%s</isSecondaryServer>
                <vpnName>%s</vpnName>
                <isPublicNet>%s</isPublicNet>
              </hwTacSrvCfg>
            </hwTacSrvCfgs>
          </hwTacTempCfg>
        </hwTacTempCfgs>
      </hwtacacs>
    </config>
"""

# delete hwtacacs server config ipv4
CE_DELETE_HWTACACS_SERVER_CFG_IPV4 = """
    <config>
      <hwtacacs xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <hwTacTempCfgs>
          <hwTacTempCfg>
            <templateName>%s</templateName>
            <hwTacSrvCfgs>
              <hwTacSrvCfg operation="delete">
                <serverIpAddress>%s</serverIpAddress>
                <serverType>%s</serverType>
                <isSecondaryServer>%s</isSecondaryServer>
                <vpnName>%s</vpnName>
                <isPublicNet>%s</isPublicNet>
              </hwTacSrvCfg>
            </hwTacSrvCfgs>
          </hwTacTempCfg>
        </hwTacTempCfgs>
      </hwtacacs>
    </config>
"""

# get hwtacacs server config ipv6
CE_GET_HWTACACS_SERVER_CFG_IPV6 = """
    <filter type="subtree">
      <hwtacacs xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <hwTacTempCfgs>
          <hwTacTempCfg>
            <templateName>%s</templateName>
            <hwTacIpv6SrvCfgs>
              <hwTacIpv6SrvCfg>
                <serverIpAddress></serverIpAddress>
                <serverType></serverType>
                <isSecondaryServer></isSecondaryServer>
                <vpnName></vpnName>
              </hwTacIpv6SrvCfg>
            </hwTacIpv6SrvCfgs>
          </hwTacTempCfg>
        </hwTacTempCfgs>
      </hwtacacs>
    </filter>
"""

# merge hwtacacs server config ipv6
CE_MERGE_HWTACACS_SERVER_CFG_IPV6 = """
    <config>
      <hwtacacs xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <hwTacTempCfgs>
          <hwTacTempCfg>
            <templateName>%s</templateName>
            <hwTacIpv6SrvCfgs>
              <hwTacIpv6SrvCfg operation="merge">
                <serverIpAddress>%s</serverIpAddress>
                <serverType>%s</serverType>
                <isSecondaryServer>%s</isSecondaryServer>
                <vpnName>%s</vpnName>
              </hwTacIpv6SrvCfg>
            </hwTacIpv6SrvCfgs>
          </hwTacTempCfg>
        </hwTacTempCfgs>
      </hwtacacs>
    </config>
"""

# delete hwtacacs server config ipv6
CE_DELETE_HWTACACS_SERVER_CFG_IPV6 = """
    <config>
      <hwtacacs xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <hwTacTempCfgs>
          <hwTacTempCfg>
            <templateName>%s</templateName>
            <hwTacIpv6SrvCfgs>
              <hwTacIpv6SrvCfg operation="delete">
                <serverIpAddress>%s</serverIpAddress>
                <serverType>%s</serverType>
                <isSecondaryServer>%s</isSecondaryServer>
                <vpnName>%s</vpnName>
              </hwTacIpv6SrvCfg>
            </hwTacIpv6SrvCfgs>
          </hwTacTempCfg>
        </hwTacTempCfgs>
      </hwtacacs>
    </config>
"""

# get hwtacacs host server config
CE_GET_HWTACACS_HOST_SERVER_CFG = """
    <filter type="subtree">
      <hwtacacs xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <hwTacTempCfgs>
          <hwTacTempCfg>
            <templateName>%s</templateName>
            <hwTacHostSrvCfgs>
              <hwTacHostSrvCfg>
                <serverHostName></serverHostName>
                <serverType></serverType>
                <isSecondaryServer></isSecondaryServer>
                <vpnName></vpnName>
                <isPublicNet></isPublicNet>
              </hwTacHostSrvCfg>
            </hwTacHostSrvCfgs>
          </hwTacTempCfg>
        </hwTacTempCfgs>
      </hwtacacs>
    </filter>
"""

# merge hwtacacs host server config
CE_MERGE_HWTACACS_HOST_SERVER_CFG = """
    <config>
      <hwtacacs xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <hwTacTempCfgs>
          <hwTacTempCfg>
            <templateName>%s</templateName>
            <hwTacHostSrvCfgs>
              <hwTacHostSrvCfg operation="merge">
                <serverHostName>%s</serverHostName>
                <serverType>%s</serverType>
                <isSecondaryServer>%s</isSecondaryServer>
                <vpnName>%s</vpnName>
                <isPublicNet>%s</isPublicNet>
              </hwTacHostSrvCfg>
            </hwTacHostSrvCfgs>
          </hwTacTempCfg>
        </hwTacTempCfgs>
      </hwtacacs>
    </config>
"""

# delete hwtacacs host server config
CE_DELETE_HWTACACS_HOST_SERVER_CFG = """
    <config>
      <hwtacacs xmlns="http://www.huawei.com/netconf/vrp" content-version="1.0" format-version="1.0">
        <hwTacTempCfgs>
          <hwTacTempCfg>
            <templateName>%s</templateName>
            <hwTacHostSrvCfgs>
              <hwTacHostSrvCfg operation="delete">
                <serverHostName>%s</serverHostName>
                <serverType>%s</serverType>
                <isSecondaryServer>%s</isSecondaryServer>
                <vpnName>%s</vpnName>
                <isPublicNet>%s</isPublicNet>
              </hwTacHostSrvCfg>
            </hwTacHostSrvCfgs>
          </hwTacTempCfg>
        </hwTacTempCfgs>
      </hwtacacs>
    </config>
"""


class AaaServerHost(object):
    """ Manages aaa server host configuration """

    def netconf_get_config(self, **kwargs):
        """ Get configure by netconf """

        module = kwargs["module"]
        conf_str = kwargs["conf_str"]

        xml_str = get_nc_config(module, conf_str)

        return xml_str

    def netconf_set_config(self, **kwargs):
        """ Set configure by netconf """

        module = kwargs["module"]
        conf_str = kwargs["conf_str"]

        recv_xml = set_nc_config(module, conf_str)

        return recv_xml

    def get_local_user_info(self, **kwargs):
        """ Get local user information """

        module = kwargs["module"]
        local_user_name = module.params['local_user_name']
        local_service_type = module.params['local_service_type']
        local_ftp_dir = module.params['local_ftp_dir']
        local_user_level = module.params['local_user_level']
        local_user_group = module.params['local_user_group']
        state = module.params['state']

        result = dict()
        result["local_user_info"] = []
        need_cfg = False

        conf_str = CE_GET_LOCAL_USER_INFO_HEADER

        if local_service_type:
            if local_service_type == "none":
                conf_str += "<serviceTerminal></serviceTerminal>"
                conf_str += "<serviceTelnet></serviceTelnet>"
                conf_str += "<serviceFtp></serviceFtp>"
                conf_str += "<serviceSsh></serviceSsh>"
                conf_str += "<serviceSnmp></serviceSnmp>"
                conf_str += "<serviceDot1x></serviceDot1x>"
            elif local_service_type == "dot1x":
                conf_str += "<serviceDot1x></serviceDot1x>"
            else:
                option = local_service_type.split(" ")
                for tmp in option:
                    if tmp == "dot1x":
                        module.fail_json(
                            msg='Error: Do not input dot1x with other service type.')
                    elif tmp == "none":
                        module.fail_json(
                            msg='Error: Do not input none with other service type.')
                    elif tmp == "ftp":
                        conf_str += "<serviceFtp></serviceFtp>"
                    elif tmp == "snmp":
                        conf_str += "<serviceSnmp></serviceSnmp>"
                    elif tmp == "ssh":
                        conf_str += "<serviceSsh></serviceSsh>"
                    elif tmp == "telnet":
                        conf_str += "<serviceTelnet></serviceTelnet>"
                    elif tmp == "terminal":
                        conf_str += "<serviceTerminal></serviceTerminal>"
                    else:
                        module.fail_json(
                            msg='Error: Do not support the type [%s].' % tmp)

        if local_ftp_dir:
            conf_str += "<ftpDir></ftpDir>"

        if local_user_level:
            conf_str += "<userLevel></userLevel>"

        if local_user_group:
            conf_str += "<userGroupName></userGroupName>"

        conf_str += CE_GET_LOCAL_USER_INFO_TAIL

        recv_xml = self.netconf_get_config(module=module, conf_str=conf_str)

        if "<data/>" in recv_xml:
            if state == "present":
                need_cfg = True

        else:
            xml_str = recv_xml.replace('\r', '').replace('\n', '').\
                replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "").\
                replace('xmlns="http://www.huawei.com/netconf/vrp"', "")

            root = ElementTree.fromstring(xml_str)
            local_user_info = root.findall("aaa/lam/users/user")
            if local_user_info:
                for tmp in local_user_info:
                    tmp_dict = dict()
                    for site in tmp:
                        if site.tag in ["userName", "password", "userLevel", "ftpDir", "userGroupName",
                                        "serviceTerminal", "serviceTelnet", "serviceFtp", "serviceSsh",
                                        "serviceSnmp", "serviceDot1x"]:
                            tmp_dict[site.tag] = site.text

                    result["local_user_info"].append(tmp_dict)

            if state == "present":
                need_cfg = True
            else:
                if result["local_user_info"]:
                    for tmp in result["local_user_info"]:
                        if "userName" in tmp.keys():
                            if tmp["userName"] == local_user_name:

                                if not local_service_type and not local_user_level \
                                        and not local_ftp_dir and not local_user_group:

                                    need_cfg = True

                                if local_service_type:
                                    if local_service_type == "none":
                                        if tmp.get("serviceTerminal") == "true" or \
                                                tmp.get("serviceTelnet") == "true" or \
                                                tmp.get("serviceFtp") == "true" or \
                                                tmp.get("serviceSsh") == "true" or \
                                                tmp.get("serviceSnmp") == "true" or \
                                                tmp.get("serviceDot1x") == "true":
                                            need_cfg = True
                                    elif local_service_type == "dot1x":
                                        if tmp.get("serviceDot1x") == "true":
                                            need_cfg = True
                                    elif tmp == "ftp":
                                        if tmp.get("serviceFtp") == "true":
                                            need_cfg = True
                                    elif tmp == "snmp":
                                        if tmp.get("serviceSnmp") == "true":
                                            need_cfg = True
                                    elif tmp == "ssh":
                                        if tmp.get("serviceSsh") == "true":
                                            need_cfg = True
                                    elif tmp == "telnet":
                                        if tmp.get("serviceTelnet") == "true":
                                            need_cfg = True
                                    elif tmp == "terminal":
                                        if tmp.get("serviceTerminal") == "true":
                                            need_cfg = True

                                if local_user_level:
                                    if tmp.get("userLevel") == local_user_level:
                                        need_cfg = True

                                if local_ftp_dir:
                                    if tmp.get("ftpDir") == local_ftp_dir:
                                        need_cfg = True

                                if local_user_group:
                                    if tmp.get("userGroupName") == local_user_group:
                                        need_cfg = True

                                break

        result["need_cfg"] = need_cfg
        return result

    def merge_local_user_info(self, **kwargs):
        """ Merge local user information by netconf """

        module = kwargs["module"]
        local_user_name = module.params['local_user_name']
        local_password = module.params['local_password']
        local_service_type = module.params['local_service_type']
        local_ftp_dir = module.params['local_ftp_dir']
        local_user_level = module.params['local_user_level']
        local_user_group = module.params['local_user_group']
        state = module.params['state']

        cmds = []

        conf_str = CE_MERGE_LOCAL_USER_INFO_HEADER % local_user_name

        if local_password:
            conf_str += "<password>%s</password>" % local_password

        if state == "present":
            cmd = "local-user %s password cipher %s" % (
                local_user_name, local_password)
            cmds.append(cmd)

        if local_service_type:
            if local_service_type == "none":
                conf_str += "<serviceTerminal>false</serviceTerminal>"
                conf_str += "<serviceTelnet>false</serviceTelnet>"
                conf_str += "<serviceFtp>false</serviceFtp>"
                conf_str += "<serviceSsh>false</serviceSsh>"
                conf_str += "<serviceSnmp>false</serviceSnmp>"
                conf_str += "<serviceDot1x>false</serviceDot1x>"

                cmd = "local-user %s service-type none" % local_user_name
                cmds.append(cmd)

            elif local_service_type == "dot1x":
                if state == "present":
                    conf_str += "<serviceDot1x>true</serviceDot1x>"
                    cmd = "local-user %s service-type dot1x" % local_user_name
                else:
                    conf_str += "<serviceDot1x>false</serviceDot1x>"
                    cmd = "undo local-user %s service-type" % local_user_name

                cmds.append(cmd)

            else:
                option = local_service_type.split(" ")
                for tmp in option:
                    if tmp == "dot1x":
                        module.fail_json(
                            msg='Error: Do not input dot1x with other service type.')
                    if tmp == "none":
                        module.fail_json(
                            msg='Error: Do not input none with other service type.')

                    if state == "present":
                        if tmp == "ftp":
                            conf_str += "<serviceFtp>true</serviceFtp>"
                            cmd = "local-user %s service-type ftp" % local_user_name
                        elif tmp == "snmp":
                            conf_str += "<serviceSnmp>true</serviceSnmp>"
                            cmd = "local-user %s service-type snmp" % local_user_name
                        elif tmp == "ssh":
                            conf_str += "<serviceSsh>true</serviceSsh>"
                            cmd = "local-user %s service-type ssh" % local_user_name
                        elif tmp == "telnet":
                            conf_str += "<serviceTelnet>true</serviceTelnet>"
                            cmd = "local-user %s service-type telnet" % local_user_name
                        elif tmp == "terminal":
                            conf_str += "<serviceTerminal>true</serviceTerminal>"
                            cmd = "local-user %s service-type terminal" % local_user_name

                        cmds.append(cmd)

                    else:
                        if tmp == "ftp":
                            conf_str += "<serviceFtp>false</serviceFtp>"
                        elif tmp == "snmp":
                            conf_str += "<serviceSnmp>false</serviceSnmp>"
                        elif tmp == "ssh":
                            conf_str += "<serviceSsh>false</serviceSsh>"
                        elif tmp == "telnet":
                            conf_str += "<serviceTelnet>false</serviceTelnet>"
                        elif tmp == "terminal":
                            conf_str += "<serviceTerminal>false</serviceTerminal>"

                if state == "absent":
                    cmd = "undo local-user %s service-type" % local_user_name
                    cmds.append(cmd)

        if local_ftp_dir:
            if state == "present":
                conf_str += "<ftpDir>%s</ftpDir>" % local_ftp_dir
                cmd = "local-user %s ftp-directory %s" % (
                    local_user_name, local_ftp_dir)
                cmds.append(cmd)
            else:
                conf_str += "<ftpDir></ftpDir>"
                cmd = "undo local-user %s ftp-directory" % local_user_name
                cmds.append(cmd)

        if local_user_level:
            if state == "present":
                conf_str += "<userLevel>%s</userLevel>" % local_user_level
                cmd = "local-user %s level %s" % (
                    local_user_name, local_user_level)
                cmds.append(cmd)
            else:
                conf_str += "<userLevel></userLevel>"
                cmd = "undo local-user %s level" % local_user_name
                cmds.append(cmd)

        if local_user_group:
            if state == "present":
                conf_str += "<userGroupName>%s</userGroupName>" % local_user_group
                cmd = "local-user %s user-group %s" % (
                    local_user_name, local_user_group)
                cmds.append(cmd)
            else:
                conf_str += "<userGroupName></userGroupName>"
                cmd = "undo local-user %s user-group" % local_user_name
                cmds.append(cmd)

        conf_str += CE_MERGE_LOCAL_USER_INFO_TAIL

        recv_xml = self.netconf_set_config(module=module, conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            module.fail_json(msg='Error: Merge local user info failed.')

        return cmds

    def delete_local_user_info(self, **kwargs):
        """ Delete local user information by netconf """

        module = kwargs["module"]
        local_user_name = module.params['local_user_name']
        conf_str = CE_DELETE_LOCAL_USER_INFO_HEADER % local_user_name
        conf_str += CE_DELETE_LOCAL_USER_INFO_TAIL

        cmds = []

        recv_xml = self.netconf_set_config(module=module, conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            module.fail_json(msg='Error: Delete local user info failed.')

        cmd = "undo local-user %s" % local_user_name
        cmds.append(cmd)

        return cmds

    def get_radius_server_cfg_ipv4(self, **kwargs):
        """ Get radius server configure ipv4 """

        module = kwargs["module"]
        radius_group_name = module.params['radius_group_name']
        radius_server_type = module.params['radius_server_type']
        radius_server_ip = module.params['radius_server_ip']
        radius_server_port = module.params['radius_server_port']
        radius_server_mode = module.params['radius_server_mode']
        radius_vpn_name = module.params['radius_vpn_name']
        state = module.params['state']

        result = dict()
        result["radius_server_ip_v4"] = []
        need_cfg = False

        conf_str = CE_GET_RADIUS_SERVER_CFG_IPV4 % radius_group_name

        recv_xml = self.netconf_get_config(module=module, conf_str=conf_str)

        if "<data/>" in recv_xml:
            if state == "present":
                need_cfg = True

        else:
            xml_str = recv_xml.replace('\r', '').replace('\n', '').\
                replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "").\
                replace('xmlns="http://www.huawei.com/netconf/vrp"', "")

            root = ElementTree.fromstring(xml_str)
            radius_server_ip_v4 = root.findall(
                "radius/rdsTemplates/rdsTemplate/rdsServerIPV4s/rdsServerIPV4")
            if radius_server_ip_v4:
                for tmp in radius_server_ip_v4:
                    tmp_dict = dict()
                    for site in tmp:
                        if site.tag in ["serverType", "serverIPAddress", "serverPort", "serverMode", "vpnName"]:
                            tmp_dict[site.tag] = site.text

                    result["radius_server_ip_v4"].append(tmp_dict)

            if result["radius_server_ip_v4"]:
                cfg = dict()
                config_list = list()
                if radius_server_type:
                    cfg["serverType"] = radius_server_type.lower()
                if radius_server_ip:
                    cfg["serverIPAddress"] = radius_server_ip.lower()
                if radius_server_port:
                    cfg["serverPort"] = radius_server_port.lower()
                if radius_server_mode:
                    cfg["serverMode"] = radius_server_mode.lower()
                if radius_vpn_name:
                    cfg["vpnName"] = radius_vpn_name.lower()

                for tmp in result["radius_server_ip_v4"]:
                    exist_cfg = dict()
                    if radius_server_type:
                        exist_cfg["serverType"] = tmp.get("serverType").lower()
                    if radius_server_ip:
                        exist_cfg["serverIPAddress"] = tmp.get("serverIPAddress").lower()
                    if radius_server_port:
                        exist_cfg["serverPort"] = tmp.get("serverPort").lower()
                    if radius_server_mode:
                        exist_cfg["serverMode"] = tmp.get("serverMode").lower()
                    if radius_vpn_name:
                        exist_cfg["vpnName"] = tmp.get("vpnName").lower()
                    config_list.append(exist_cfg)
                if cfg in config_list:
                    if state == "present":
                        need_cfg = False
                    else:
                        need_cfg = True
                else:
                    if state == "present":
                        need_cfg = True
                    else:
                        need_cfg = False
        result["need_cfg"] = need_cfg
        return result

    def merge_radius_server_cfg_ipv4(self, **kwargs):
        """ Merge radius server configure ipv4 """

        module = kwargs["module"]
        radius_group_name = module.params['radius_group_name']
        radius_server_type = module.params['radius_server_type']
        radius_server_ip = module.params['radius_server_ip']
        radius_server_port = module.params['radius_server_port']
        radius_server_mode = module.params['radius_server_mode']
        radius_vpn_name = module.params['radius_vpn_name']

        conf_str = CE_MERGE_RADIUS_SERVER_CFG_IPV4 % (
            radius_group_name, radius_server_type,
            radius_server_ip, radius_server_port,
            radius_server_mode, radius_vpn_name)

        recv_xml = self.netconf_set_config(module=module, conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            module.fail_json(
                msg='Error: Merge radius server config ipv4 failed.')

        cmds = []

        cmd = "radius server group %s" % radius_group_name
        cmds.append(cmd)

        if radius_server_type == "Authentication":
            cmd = "radius server authentication %s %s" % (
                radius_server_ip, radius_server_port)

            if radius_vpn_name and radius_vpn_name != "_public_":
                cmd += " vpn-instance %s" % radius_vpn_name

            if radius_server_mode == "Secondary-server":
                cmd += " secondary"
        else:
            cmd = "radius server accounting %s %s" % (
                radius_server_ip, radius_server_port)

            if radius_vpn_name and radius_vpn_name != "_public_":
                cmd += " vpn-instance %s" % radius_vpn_name

            if radius_server_mode == "Secondary-server":
                cmd += " secondary"

        cmds.append(cmd)
        return cmds

    def delete_radius_server_cfg_ipv4(self, **kwargs):
        """ Delete radius server configure ipv4 """

        module = kwargs["module"]
        radius_group_name = module.params['radius_group_name']
        radius_server_type = module.params['radius_server_type']
        radius_server_ip = module.params['radius_server_ip']
        radius_server_port = module.params['radius_server_port']
        radius_server_mode = module.params['radius_server_mode']
        radius_vpn_name = module.params['radius_vpn_name']

        conf_str = CE_DELETE_RADIUS_SERVER_CFG_IPV4 % (
            radius_group_name, radius_server_type,
            radius_server_ip, radius_server_port,
            radius_server_mode, radius_vpn_name)

        recv_xml = self.netconf_set_config(module=module, conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            module.fail_json(
                msg='Error: Create radius server config ipv4 failed.')

        cmds = []

        cmd = "radius server group %s" % radius_group_name
        cmds.append(cmd)

        if radius_server_type == "Authentication":
            cmd = "undo radius server authentication %s %s" % (
                radius_server_ip, radius_server_port)

            if radius_vpn_name and radius_vpn_name != "_public_":
                cmd += " vpn-instance %s" % radius_vpn_name

            if radius_server_mode == "Secondary-server":
                cmd += " secondary"
        else:
            cmd = "undo radius server accounting  %s %s" % (
                radius_server_ip, radius_server_port)

            if radius_vpn_name and radius_vpn_name != "_public_":
                cmd += " vpn-instance %s" % radius_vpn_name

            if radius_server_mode == "Secondary-server":
                cmd += " secondary"

        cmds.append(cmd)
        return cmds

    def get_radius_server_cfg_ipv6(self, **kwargs):
        """ Get radius server configure ipv6 """

        module = kwargs["module"]
        radius_group_name = module.params['radius_group_name']
        radius_server_type = module.params['radius_server_type']
        radius_server_ipv6 = module.params['radius_server_ipv6']
        radius_server_port = module.params['radius_server_port']
        radius_server_mode = module.params['radius_server_mode']
        state = module.params['state']

        result = dict()
        result["radius_server_ip_v6"] = []
        need_cfg = False

        conf_str = CE_GET_RADIUS_SERVER_CFG_IPV6 % radius_group_name

        recv_xml = self.netconf_get_config(module=module, conf_str=conf_str)

        if "<data/>" in recv_xml:
            if state == "present":
                need_cfg = True

        else:
            xml_str = recv_xml.replace('\r', '').replace('\n', '').\
                replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "").\
                replace('xmlns="http://www.huawei.com/netconf/vrp"', "")

            root = ElementTree.fromstring(xml_str)
            radius_server_ip_v6 = root.findall(
                "radius/rdsTemplates/rdsTemplate/rdsServerIPV6s/rdsServerIPV6")
            if radius_server_ip_v6:
                for tmp in radius_server_ip_v6:
                    tmp_dict = dict()
                    for site in tmp:
                        if site.tag in ["serverType", "serverIPAddress", "serverPort", "serverMode"]:
                            tmp_dict[site.tag] = site.text

                    result["radius_server_ip_v6"].append(tmp_dict)

            if result["radius_server_ip_v6"]:
                cfg = dict()
                config_list = list()
                if radius_server_type:
                    cfg["serverType"] = radius_server_type.lower()
                if radius_server_ipv6:
                    cfg["serverIPAddress"] = radius_server_ipv6.lower()
                if radius_server_port:
                    cfg["serverPort"] = radius_server_port.lower()
                if radius_server_mode:
                    cfg["serverMode"] = radius_server_mode.lower()

                for tmp in result["radius_server_ip_v6"]:
                    exist_cfg = dict()
                    if radius_server_type:
                        exist_cfg["serverType"] = tmp.get("serverType").lower()
                    if radius_server_ipv6:
                        exist_cfg["serverIPAddress"] = tmp.get("serverIPAddress").lower()
                    if radius_server_port:
                        exist_cfg["serverPort"] = tmp.get("serverPort").lower()
                    if radius_server_mode:
                        exist_cfg["serverMode"] = tmp.get("serverMode").lower()
                    config_list.append(exist_cfg)
                if cfg in config_list:
                    if state == "present":
                        need_cfg = False
                    else:
                        need_cfg = True
                else:
                    if state == "present":
                        need_cfg = True
                    else:
                        need_cfg = False

        result["need_cfg"] = need_cfg
        return result

    def merge_radius_server_cfg_ipv6(self, **kwargs):
        """ Merge radius server configure ipv6 """

        module = kwargs["module"]
        radius_group_name = module.params['radius_group_name']
        radius_server_type = module.params['radius_server_type']
        radius_server_ipv6 = module.params['radius_server_ipv6']
        radius_server_port = module.params['radius_server_port']
        radius_server_mode = module.params['radius_server_mode']

        conf_str = CE_MERGE_RADIUS_SERVER_CFG_IPV6 % (
            radius_group_name, radius_server_type,
            radius_server_ipv6, radius_server_port,
            radius_server_mode)

        recv_xml = self.netconf_set_config(module=module, conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            module.fail_json(
                msg='Error: Merge radius server config ipv6 failed.')

        cmds = []

        cmd = "radius server group %s" % radius_group_name
        cmds.append(cmd)

        if radius_server_type == "Authentication":
            cmd = "radius server authentication %s %s" % (
                radius_server_ipv6, radius_server_port)

            if radius_server_mode == "Secondary-server":
                cmd += " secondary"
        else:
            cmd = "radius server accounting  %s %s" % (
                radius_server_ipv6, radius_server_port)

            if radius_server_mode == "Secondary-server":
                cmd += " secondary"

        cmds.append(cmd)
        return cmds

    def delete_radius_server_cfg_ipv6(self, **kwargs):
        """ Delete radius server configure ipv6 """

        module = kwargs["module"]
        radius_group_name = module.params['radius_group_name']
        radius_server_type = module.params['radius_server_type']
        radius_server_ipv6 = module.params['radius_server_ipv6']
        radius_server_port = module.params['radius_server_port']
        radius_server_mode = module.params['radius_server_mode']

        conf_str = CE_DELETE_RADIUS_SERVER_CFG_IPV6 % (
            radius_group_name, radius_server_type,
            radius_server_ipv6, radius_server_port,
            radius_server_mode)

        recv_xml = self.netconf_set_config(module=module, conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            module.fail_json(
                msg='Error: Create radius server config ipv6 failed.')

        cmds = []

        cmd = "radius server group %s" % radius_group_name
        cmds.append(cmd)

        if radius_server_type == "Authentication":
            cmd = "undo radius server authentication %s %s" % (
                radius_server_ipv6, radius_server_port)

            if radius_server_mode == "Secondary-server":
                cmd += " secondary"
        else:
            cmd = "undo radius server accounting  %s %s" % (
                radius_server_ipv6, radius_server_port)

            if radius_server_mode == "Secondary-server":
                cmd += " secondary"

        cmds.append(cmd)
        return cmds

    def get_radius_server_name(self, **kwargs):
        """ Get radius server name """

        module = kwargs["module"]
        radius_group_name = module.params['radius_group_name']
        radius_server_type = module.params['radius_server_type']
        radius_server_name = module.params['radius_server_name']
        radius_server_port = module.params['radius_server_port']
        radius_server_mode = module.params['radius_server_mode']
        radius_vpn_name = module.params['radius_vpn_name']
        state = module.params['state']

        result = dict()
        result["radius_server_name_cfg"] = []
        need_cfg = False

        conf_str = CE_GET_RADIUS_SERVER_NAME % radius_group_name

        recv_xml = self.netconf_get_config(module=module, conf_str=conf_str)

        if "<data/>" in recv_xml:
            if state == "present":
                need_cfg = True

        else:
            xml_str = recv_xml.replace('\r', '').replace('\n', '').\
                replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "").\
                replace('xmlns="http://www.huawei.com/netconf/vrp"', "")

            root = ElementTree.fromstring(xml_str)
            radius_server_name_cfg = root.findall(
                "radius/rdsTemplates/rdsTemplate/rdsServerNames/rdsServerName")
            if radius_server_name_cfg:
                for tmp in radius_server_name_cfg:
                    tmp_dict = dict()
                    for site in tmp:
                        if site.tag in ["serverType", "serverName", "serverPort", "serverMode", "vpnName"]:
                            tmp_dict[site.tag] = site.text

                    result["radius_server_name_cfg"].append(tmp_dict)

            if result["radius_server_name_cfg"]:
                cfg = dict()
                config_list = list()
                if radius_server_type:
                    cfg["serverType"] = radius_server_type.lower()
                if radius_server_name:
                    cfg["serverName"] = radius_server_name.lower()
                if radius_server_port:
                    cfg["serverPort"] = radius_server_port.lower()
                if radius_server_mode:
                    cfg["serverMode"] = radius_server_mode.lower()
                if radius_vpn_name:
                    cfg["vpnName"] = radius_vpn_name.lower()

                for tmp in result["radius_server_name_cfg"]:
                    exist_cfg = dict()
                    if radius_server_type:
                        exist_cfg["serverType"] = tmp.get("serverType").lower()
                    if radius_server_name:
                        exist_cfg["serverName"] = tmp.get("serverName").lower()
                    if radius_server_port:
                        exist_cfg["serverPort"] = tmp.get("serverPort").lower()
                    if radius_server_mode:
                        exist_cfg["serverMode"] = tmp.get("serverMode").lower()
                    if radius_vpn_name:
                        exist_cfg["vpnName"] = tmp.get("vpnName").lower()
                    config_list.append(exist_cfg)
                if cfg in config_list:
                    if state == "present":
                        need_cfg = False
                    else:
                        need_cfg = True
                else:
                    if state == "present":
                        need_cfg = True
                    else:
                        need_cfg = False
        result["need_cfg"] = need_cfg
        return result

    def merge_radius_server_name(self, **kwargs):
        """ Merge radius server name """

        module = kwargs["module"]
        radius_group_name = module.params['radius_group_name']
        radius_server_type = module.params['radius_server_type']
        radius_server_name = module.params['radius_server_name']
        radius_server_port = module.params['radius_server_port']
        radius_server_mode = module.params['radius_server_mode']
        radius_vpn_name = module.params['radius_vpn_name']

        conf_str = CE_MERGE_RADIUS_SERVER_NAME % (
            radius_group_name, radius_server_type,
            radius_server_name, radius_server_port,
            radius_server_mode, radius_vpn_name)

        recv_xml = self.netconf_set_config(module=module, conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            module.fail_json(msg='Error: Merge radius server name failed.')

        cmds = []

        cmd = "radius server group %s" % radius_group_name
        cmds.append(cmd)

        if radius_server_type == "Authentication":
            cmd = "radius server authentication hostname %s %s" % (
                radius_server_name, radius_server_port)

            if radius_vpn_name and radius_vpn_name != "_public_":
                cmd += " vpn-instance %s" % radius_vpn_name

            if radius_server_mode == "Secondary-server":
                cmd += " secondary"
        else:
            cmd = "radius server accounting hostname %s %s" % (
                radius_server_name, radius_server_port)

            if radius_vpn_name and radius_vpn_name != "_public_":
                cmd += " vpn-instance %s" % radius_vpn_name

            if radius_server_mode == "Secondary-server":
                cmd += " secondary"

        cmds.append(cmd)
        return cmds

    def delete_radius_server_name(self, **kwargs):
        """ Delete radius server name """

        module = kwargs["module"]
        radius_group_name = module.params['radius_group_name']
        radius_server_type = module.params['radius_server_type']
        radius_server_name = module.params['radius_server_name']
        radius_server_port = module.params['radius_server_port']
        radius_server_mode = module.params['radius_server_mode']
        radius_vpn_name = module.params['radius_vpn_name']

        conf_str = CE_DELETE_RADIUS_SERVER_NAME % (
            radius_group_name, radius_server_type,
            radius_server_name, radius_server_port,
            radius_server_mode, radius_vpn_name)

        recv_xml = self.netconf_set_config(module=module, conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            module.fail_json(msg='Error: delete radius server name failed.')

        cmds = []

        cmd = "radius server group %s" % radius_group_name
        cmds.append(cmd)

        if radius_server_type == "Authentication":
            cmd = "undo radius server authentication hostname %s %s" % (
                radius_server_name, radius_server_port)

            if radius_vpn_name and radius_vpn_name != "_public_":
                cmd += " vpn-instance %s" % radius_vpn_name

            if radius_server_mode == "Secondary-server":
                cmd += " secondary"
        else:
            cmd = "undo radius server accounting hostname %s %s" % (
                radius_server_name, radius_server_port)

            if radius_vpn_name and radius_vpn_name != "_public_":
                cmd += " vpn-instance %s" % radius_vpn_name

            if radius_server_mode == "Secondary-server":
                cmd += " secondary"

        cmds.append(cmd)
        return cmds

    def get_hwtacacs_server_cfg_ipv4(self, **kwargs):
        """ Get hwtacacs server configure ipv4 """

        module = kwargs["module"]
        hwtacacs_template = module.params["hwtacacs_template"]
        hwtacacs_server_ip = module.params["hwtacacs_server_ip"]
        hwtacacs_server_type = module.params["hwtacacs_server_type"]
        hwtacacs_is_secondary_server = module.params[
            "hwtacacs_is_secondary_server"]
        hwtacacs_vpn_name = module.params["hwtacacs_vpn_name"]
        hwtacacs_is_public_net = module.params["hwtacacs_is_public_net"]
        state = module.params["state"]

        result = dict()
        result["hwtacacs_server_cfg_ipv4"] = []
        need_cfg = False

        conf_str = CE_GET_HWTACACS_SERVER_CFG_IPV4 % hwtacacs_template

        recv_xml = self.netconf_get_config(module=module, conf_str=conf_str)

        if "<data/>" in recv_xml:
            if state == "present":
                need_cfg = True

        else:
            xml_str = recv_xml.replace('\r', '').replace('\n', '').\
                replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "").\
                replace('xmlns="http://www.huawei.com/netconf/vrp"', "")

            root = ElementTree.fromstring(xml_str)
            hwtacacs_server_cfg_ipv4 = root.findall(
                "hwtacacs/hwTacTempCfgs/hwTacTempCfg/hwTacSrvCfgs/hwTacSrvCfg")
            if hwtacacs_server_cfg_ipv4:
                for tmp in hwtacacs_server_cfg_ipv4:
                    tmp_dict = dict()
                    for site in tmp:
                        if site.tag in ["serverIpAddress", "serverType", "isSecondaryServer", "isPublicNet", "vpnName"]:
                            tmp_dict[site.tag] = site.text

                    result["hwtacacs_server_cfg_ipv4"].append(tmp_dict)

            if result["hwtacacs_server_cfg_ipv4"]:
                cfg = dict()
                config_list = list()

                if hwtacacs_server_ip:
                    cfg["serverIpAddress"] = hwtacacs_server_ip.lower()
                if hwtacacs_server_type:
                    cfg["serverType"] = hwtacacs_server_type.lower()
                if hwtacacs_is_secondary_server:
                    cfg["isSecondaryServer"] = str(hwtacacs_is_secondary_server).lower()
                if hwtacacs_is_public_net:
                    cfg["isPublicNet"] = str(hwtacacs_is_public_net).lower()
                if hwtacacs_vpn_name:
                    cfg["vpnName"] = hwtacacs_vpn_name.lower()

                for tmp in result["hwtacacs_server_cfg_ipv4"]:
                    exist_cfg = dict()
                    if hwtacacs_server_ip:
                        exist_cfg["serverIpAddress"] = tmp.get("serverIpAddress").lower()
                    if hwtacacs_server_type:
                        exist_cfg["serverType"] = tmp.get("serverType").lower()
                    if hwtacacs_is_secondary_server:
                        exist_cfg["isSecondaryServer"] = tmp.get("isSecondaryServer").lower()
                    if hwtacacs_is_public_net:
                        exist_cfg["isPublicNet"] = tmp.get("isPublicNet").lower()
                    if hwtacacs_vpn_name:
                        exist_cfg["vpnName"] = tmp.get("vpnName").lower()
                    config_list.append(exist_cfg)
                if cfg in config_list:
                    if state == "present":
                        need_cfg = False
                    else:
                        need_cfg = True
                else:
                    if state == "present":
                        need_cfg = True
                    else:
                        need_cfg = False
        result["need_cfg"] = need_cfg
        return result

    def merge_hwtacacs_server_cfg_ipv4(self, **kwargs):
        """ Merge hwtacacs server configure ipv4 """

        module = kwargs["module"]
        hwtacacs_template = module.params["hwtacacs_template"]
        hwtacacs_server_ip = module.params["hwtacacs_server_ip"]
        hwtacacs_server_type = module.params["hwtacacs_server_type"]
        hwtacacs_is_secondary_server = module.params[
            "hwtacacs_is_secondary_server"]
        hwtacacs_vpn_name = module.params["hwtacacs_vpn_name"]
        hwtacacs_is_public_net = module.params["hwtacacs_is_public_net"]

        conf_str = CE_MERGE_HWTACACS_SERVER_CFG_IPV4 % (
            hwtacacs_template, hwtacacs_server_ip,
            hwtacacs_server_type, str(hwtacacs_is_secondary_server).lower(),
            hwtacacs_vpn_name, str(hwtacacs_is_public_net).lower())

        recv_xml = self.netconf_set_config(module=module, conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            module.fail_json(
                msg='Error: Merge hwtacacs server config ipv4 failed.')

        cmds = []

        cmd = "hwtacacs server template %s" % hwtacacs_template
        cmds.append(cmd)

        if hwtacacs_server_type == "Authentication":
            cmd = "hwtacacs server authentication %s" % hwtacacs_server_ip
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Authorization":
            cmd = "hwtacacs server authorization %s" % hwtacacs_server_ip
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Accounting":
            cmd = "hwtacacs server accounting %s" % hwtacacs_server_ip
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Common":
            cmd = "hwtacacs server %s" % hwtacacs_server_ip
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        cmds.append(cmd)
        return cmds

    def delete_hwtacacs_server_cfg_ipv4(self, **kwargs):
        """ Delete hwtacacs server configure ipv4 """

        module = kwargs["module"]
        hwtacacs_template = module.params["hwtacacs_template"]
        hwtacacs_server_ip = module.params["hwtacacs_server_ip"]
        hwtacacs_server_type = module.params["hwtacacs_server_type"]
        hwtacacs_is_secondary_server = module.params[
            "hwtacacs_is_secondary_server"]
        hwtacacs_vpn_name = module.params["hwtacacs_vpn_name"]
        hwtacacs_is_public_net = module.params["hwtacacs_is_public_net"]

        conf_str = CE_DELETE_HWTACACS_SERVER_CFG_IPV4 % (
            hwtacacs_template, hwtacacs_server_ip,
            hwtacacs_server_type, str(hwtacacs_is_secondary_server).lower(),
            hwtacacs_vpn_name, str(hwtacacs_is_public_net).lower())

        recv_xml = self.netconf_set_config(module=module, conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            module.fail_json(
                msg='Error: Delete hwtacacs server config ipv4 failed.')

        cmds = []

        cmd = "hwtacacs server template %s" % hwtacacs_template
        cmds.append(cmd)

        if hwtacacs_server_type == "Authentication":
            cmd = "undo hwtacacs server authentication %s" % hwtacacs_server_ip
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Authorization":
            cmd = "undo hwtacacs server authorization %s" % hwtacacs_server_ip
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Accounting":
            cmd = "undo hwtacacs server accounting %s" % hwtacacs_server_ip
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Common":
            cmd = "undo hwtacacs server %s" % hwtacacs_server_ip
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        cmds.append(cmd)
        return cmds

    def get_hwtacacs_server_cfg_ipv6(self, **kwargs):
        """ Get hwtacacs server configure ipv6 """

        module = kwargs["module"]
        hwtacacs_template = module.params["hwtacacs_template"]
        hwtacacs_server_ipv6 = module.params["hwtacacs_server_ipv6"]
        hwtacacs_server_type = module.params["hwtacacs_server_type"]
        hwtacacs_is_secondary_server = module.params[
            "hwtacacs_is_secondary_server"]
        hwtacacs_vpn_name = module.params["hwtacacs_vpn_name"]
        state = module.params["state"]

        result = dict()
        result["hwtacacs_server_cfg_ipv6"] = []
        need_cfg = False

        conf_str = CE_GET_HWTACACS_SERVER_CFG_IPV6 % hwtacacs_template

        recv_xml = self.netconf_get_config(module=module, conf_str=conf_str)

        if "<data/>" in recv_xml:
            if state == "present":
                need_cfg = True

        else:
            xml_str = recv_xml.replace('\r', '').replace('\n', '').\
                replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "").\
                replace('xmlns="http://www.huawei.com/netconf/vrp"', "")

            root = ElementTree.fromstring(xml_str)
            hwtacacs_server_cfg_ipv6 = root.findall(
                "hwtacacs/hwTacTempCfgs/hwTacTempCfg/hwTacIpv6SrvCfgs/hwTacIpv6SrvCfg")
            if hwtacacs_server_cfg_ipv6:
                for tmp in hwtacacs_server_cfg_ipv6:
                    tmp_dict = dict()
                    for site in tmp:
                        if site.tag in ["serverIpAddress", "serverType", "isSecondaryServer", "vpnName"]:
                            tmp_dict[site.tag] = site.text

                    result["hwtacacs_server_cfg_ipv6"].append(tmp_dict)

            if result["hwtacacs_server_cfg_ipv6"]:
                cfg = dict()
                config_list = list()

                if hwtacacs_server_ipv6:
                    cfg["serverIpAddress"] = hwtacacs_server_ipv6.lower()
                if hwtacacs_server_type:
                    cfg["serverType"] = hwtacacs_server_type.lower()
                if hwtacacs_is_secondary_server:
                    cfg["isSecondaryServer"] = str(hwtacacs_is_secondary_server).lower()
                if hwtacacs_vpn_name:
                    cfg["vpnName"] = hwtacacs_vpn_name.lower()

                for tmp in result["hwtacacs_server_cfg_ipv6"]:
                    exist_cfg = dict()
                    if hwtacacs_server_ipv6:
                        exist_cfg["serverIpAddress"] = tmp.get("serverIpAddress").lower()
                    if hwtacacs_server_type:
                        exist_cfg["serverType"] = tmp.get("serverType").lower()
                    if hwtacacs_is_secondary_server:
                        exist_cfg["isSecondaryServer"] = tmp.get("isSecondaryServer").lower()
                    if hwtacacs_vpn_name:
                        exist_cfg["vpnName"] = tmp.get("vpnName").lower()
                    config_list.append(exist_cfg)
                if cfg in config_list:
                    if state == "present":
                        need_cfg = False
                    else:
                        need_cfg = True
                else:
                    if state == "present":
                        need_cfg = True
                    else:
                        need_cfg = False
        result["need_cfg"] = need_cfg
        return result

    def merge_hwtacacs_server_cfg_ipv6(self, **kwargs):
        """ Merge hwtacacs server configure ipv6 """

        module = kwargs["module"]
        hwtacacs_template = module.params["hwtacacs_template"]
        hwtacacs_server_ipv6 = module.params["hwtacacs_server_ipv6"]
        hwtacacs_server_type = module.params["hwtacacs_server_type"]
        hwtacacs_is_secondary_server = module.params[
            "hwtacacs_is_secondary_server"]
        hwtacacs_vpn_name = module.params["hwtacacs_vpn_name"]

        conf_str = CE_MERGE_HWTACACS_SERVER_CFG_IPV6 % (
            hwtacacs_template, hwtacacs_server_ipv6,
            hwtacacs_server_type, str(hwtacacs_is_secondary_server).lower(),
            hwtacacs_vpn_name)

        recv_xml = self.netconf_set_config(module=module, conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            module.fail_json(
                msg='Error: Merge hwtacacs server config ipv6 failed.')

        cmds = []

        cmd = "hwtacacs server template %s" % hwtacacs_template
        cmds.append(cmd)

        if hwtacacs_server_type == "Authentication":
            cmd = "hwtacacs server authentication %s" % hwtacacs_server_ipv6
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Authorization":
            cmd = "hwtacacs server authorization %s" % hwtacacs_server_ipv6
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Accounting":
            cmd = "hwtacacs server accounting %s" % hwtacacs_server_ipv6
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Common":
            cmd = "hwtacacs server %s" % hwtacacs_server_ipv6
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        cmds.append(cmd)
        return cmds

    def delete_hwtacacs_server_cfg_ipv6(self, **kwargs):
        """ Delete hwtacacs server configure ipv6 """

        module = kwargs["module"]
        hwtacacs_template = module.params["hwtacacs_template"]
        hwtacacs_server_ipv6 = module.params["hwtacacs_server_ipv6"]
        hwtacacs_server_type = module.params["hwtacacs_server_type"]
        hwtacacs_is_secondary_server = module.params[
            "hwtacacs_is_secondary_server"]
        hwtacacs_vpn_name = module.params["hwtacacs_vpn_name"]

        conf_str = CE_DELETE_HWTACACS_SERVER_CFG_IPV6 % (
            hwtacacs_template, hwtacacs_server_ipv6,
            hwtacacs_server_type, str(hwtacacs_is_secondary_server).lower(),
            hwtacacs_vpn_name)

        recv_xml = self.netconf_set_config(module=module, conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            module.fail_json(
                msg='Error: Delete hwtacacs server config ipv6 failed.')

        cmds = []

        cmd = "hwtacacs server template %s" % hwtacacs_template
        cmds.append(cmd)

        if hwtacacs_server_type == "Authentication":
            cmd = "undo hwtacacs server authentication %s" % hwtacacs_server_ipv6
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Authorization":
            cmd = "undo hwtacacs server authorization %s" % hwtacacs_server_ipv6
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Accounting":
            cmd = "undo hwtacacs server accounting %s" % hwtacacs_server_ipv6
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Common":
            cmd = "undo hwtacacs server %s" % hwtacacs_server_ipv6
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        cmds.append(cmd)
        return cmds

    def get_hwtacacs_host_server_cfg(self, **kwargs):
        """ Get hwtacacs host server configure """

        module = kwargs["module"]
        hwtacacs_template = module.params["hwtacacs_template"]
        hwtacacs_server_host_name = module.params["hwtacacs_server_host_name"]
        hwtacacs_server_type = module.params["hwtacacs_server_type"]
        hwtacacs_is_secondary_server = "true" if module.params[
            "hwtacacs_is_secondary_server"] is True else "false"
        hwtacacs_vpn_name = module.params["hwtacacs_vpn_name"]
        hwtacacs_is_public_net = "true" if module.params[
            "hwtacacs_is_public_net"] is True else "false"
        state = module.params["state"]

        result = dict()
        result["hwtacacs_server_name_cfg"] = []
        need_cfg = False

        conf_str = CE_GET_HWTACACS_HOST_SERVER_CFG % hwtacacs_template

        recv_xml = self.netconf_get_config(module=module, conf_str=conf_str)

        if "<data/>" in recv_xml:
            if state == "present":
                need_cfg = True

        else:
            xml_str = recv_xml.replace('\r', '').replace('\n', '').\
                replace('xmlns="urn:ietf:params:xml:ns:netconf:base:1.0"', "").\
                replace('xmlns="http://www.huawei.com/netconf/vrp"', "")

            root = ElementTree.fromstring(xml_str)
            hwtacacs_server_name_cfg = root.findall(
                "hwtacacs/hwTacTempCfgs/hwTacTempCfg/hwTacHostSrvCfgs/hwTacHostSrvCfg")
            if hwtacacs_server_name_cfg:
                for tmp in hwtacacs_server_name_cfg:
                    tmp_dict = dict()
                    for site in tmp:
                        if site.tag in ["serverHostName", "serverType", "isSecondaryServer", "isPublicNet", "vpnName"]:
                            tmp_dict[site.tag] = site.text

                    result["hwtacacs_server_name_cfg"].append(tmp_dict)

            if result["hwtacacs_server_name_cfg"]:
                cfg = dict()
                config_list = list()

                if hwtacacs_server_host_name:
                    cfg["serverHostName"] = hwtacacs_server_host_name.lower()
                if hwtacacs_server_type:
                    cfg["serverType"] = hwtacacs_server_type.lower()
                if hwtacacs_is_secondary_server:
                    cfg["isSecondaryServer"] = str(hwtacacs_is_secondary_server).lower()
                if hwtacacs_is_public_net:
                    cfg["isPublicNet"] = str(hwtacacs_is_public_net).lower()
                if hwtacacs_vpn_name:
                    cfg["vpnName"] = hwtacacs_vpn_name.lower()

                for tmp in result["hwtacacs_server_name_cfg"]:
                    exist_cfg = dict()
                    if hwtacacs_server_host_name:
                        exist_cfg["serverHostName"] = tmp.get("serverHostName").lower()
                    if hwtacacs_server_type:
                        exist_cfg["serverType"] = tmp.get("serverType").lower()
                    if hwtacacs_is_secondary_server:
                        exist_cfg["isSecondaryServer"] = tmp.get("isSecondaryServer").lower()
                    if hwtacacs_is_public_net:
                        exist_cfg["isPublicNet"] = tmp.get("isPublicNet").lower()
                    if hwtacacs_vpn_name:
                        exist_cfg["vpnName"] = tmp.get("vpnName").lower()
                    config_list.append(exist_cfg)
                if cfg in config_list:
                    if state == "present":
                        need_cfg = False
                    else:
                        need_cfg = True
                else:
                    if state == "present":
                        need_cfg = True
                    else:
                        need_cfg = False
        result["need_cfg"] = need_cfg
        return result

    def merge_hwtacacs_host_server_cfg(self, **kwargs):
        """ Merge hwtacacs host server configure """

        module = kwargs["module"]
        hwtacacs_template = module.params["hwtacacs_template"]
        hwtacacs_server_host_name = module.params["hwtacacs_server_host_name"]
        hwtacacs_server_type = module.params["hwtacacs_server_type"]
        hwtacacs_is_secondary_server = module.params[
            "hwtacacs_is_secondary_server"]
        hwtacacs_vpn_name = module.params["hwtacacs_vpn_name"]
        hwtacacs_is_public_net = module.params["hwtacacs_is_public_net"]

        conf_str = CE_MERGE_HWTACACS_HOST_SERVER_CFG % (
            hwtacacs_template, hwtacacs_server_host_name,
            hwtacacs_server_type, str(hwtacacs_is_secondary_server).lower(),
            hwtacacs_vpn_name, str(hwtacacs_is_public_net).lower())

        recv_xml = self.netconf_set_config(module=module, conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            module.fail_json(
                msg='Error: Merge hwtacacs host server config failed.')

        cmds = []

        if hwtacacs_server_type == "Authentication":
            cmd = "hwtacacs server authentication host %s" % hwtacacs_server_host_name
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Authorization":
            cmd = "hwtacacs server authorization host %s" % hwtacacs_server_host_name
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Accounting":
            cmd = "hwtacacs server accounting host %s" % hwtacacs_server_host_name
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Common":
            cmd = "hwtacacs server host host-name %s" % hwtacacs_server_host_name
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        cmds.append(cmd)
        return cmds

    def delete_hwtacacs_host_server_cfg(self, **kwargs):
        """ Delete hwtacacs host server configure """

        module = kwargs["module"]
        hwtacacs_template = module.params["hwtacacs_template"]
        hwtacacs_server_host_name = module.params["hwtacacs_server_host_name"]
        hwtacacs_server_type = module.params["hwtacacs_server_type"]
        hwtacacs_is_secondary_server = module.params[
            "hwtacacs_is_secondary_server"]
        hwtacacs_vpn_name = module.params["hwtacacs_vpn_name"]
        hwtacacs_is_public_net = module.params["hwtacacs_is_public_net"]

        conf_str = CE_DELETE_HWTACACS_HOST_SERVER_CFG % (
            hwtacacs_template, hwtacacs_server_host_name,
            hwtacacs_server_type, str(hwtacacs_is_secondary_server).lower(),
            hwtacacs_vpn_name, str(hwtacacs_is_public_net).lower())

        recv_xml = self.netconf_set_config(module=module, conf_str=conf_str)

        if "<ok/>" not in recv_xml:
            module.fail_json(
                msg='Error: Delete hwtacacs host server config failed.')

        cmds = []

        if hwtacacs_server_type == "Authentication":
            cmd = "undo hwtacacs server authentication host %s" % hwtacacs_server_host_name
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Authorization":
            cmd = "undo hwtacacs server authorization host %s" % hwtacacs_server_host_name
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Accounting":
            cmd = "undo hwtacacs server accounting host %s" % hwtacacs_server_host_name
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        elif hwtacacs_server_type == "Common":
            cmd = "undo hwtacacs server host %s" % hwtacacs_server_host_name
            if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
                cmd += " vpn-instance %s" % hwtacacs_vpn_name
            if hwtacacs_is_public_net:
                cmd += " public-net"
            if hwtacacs_is_secondary_server:
                cmd += " secondary"

        cmds.append(cmd)
        return cmds


def check_name(**kwargs):
    """ Check invalid name """

    module = kwargs["module"]
    name = kwargs["name"]
    invalid_char = kwargs["invalid_char"]

    for item in invalid_char:
        if item in name:
            module.fail_json(
                msg='Error: Invalid char %s is in the name %s ' % (item, name))


def check_module_argument(**kwargs):
    """ Check module argument """

    module = kwargs["module"]

    # local para
    local_user_name = module.params['local_user_name']
    local_password = module.params['local_password']
    local_ftp_dir = module.params['local_ftp_dir']
    local_user_level = module.params['local_user_level']
    local_user_group = module.params['local_user_group']

    # radius para
    radius_group_name = module.params['radius_group_name']
    radius_server_ip = module.params['radius_server_ip']
    radius_server_port = module.params['radius_server_port']
    radius_vpn_name = module.params['radius_vpn_name']
    radius_server_name = module.params['radius_server_name']

    # hwtacacs para
    hwtacacs_template = module.params['hwtacacs_template']
    hwtacacs_server_ip = module.params['hwtacacs_server_ip']
    hwtacacs_vpn_name = module.params['hwtacacs_vpn_name']
    hwtacacs_server_host_name = module.params['hwtacacs_server_host_name']

    if local_user_name:
        if len(local_user_name) > 253:
            module.fail_json(
                msg='Error: The local_user_name %s is large than 253.' % local_user_name)
        check_name(module=module, name=local_user_name,
                   invalid_char=INVALID_USER_NAME_CHAR)

    if local_password and len(local_password) > 255:
        module.fail_json(
            msg='Error: The local_password %s is large than 255.' % local_password)

    if local_user_level:
        if int(local_user_level) > 15 or int(local_user_level) < 0:
            module.fail_json(
                msg='Error: The local_user_level %s is out of [0 - 15].' % local_user_level)

    if local_ftp_dir:
        if len(local_ftp_dir) > 255:
            module.fail_json(
                msg='Error: The local_ftp_dir %s is large than 255.' % local_ftp_dir)

    if local_user_group:
        if len(local_user_group) > 32 or len(local_user_group) < 1:
            module.fail_json(
                msg='Error: The local_user_group %s is out of [1 - 32].' % local_user_group)

    if radius_group_name and len(radius_group_name) > 32:
        module.fail_json(
            msg='Error: The radius_group_name %s is large than 32.' % radius_group_name)

    if radius_server_ip and not check_ip_addr(radius_server_ip):
        module.fail_json(
            msg='Error: The radius_server_ip %s is invalid.' % radius_server_ip)

    if radius_server_port and not radius_server_port.isdigit():
        module.fail_json(
            msg='Error: The radius_server_port %s is invalid.' % radius_server_port)

    if radius_vpn_name:
        if len(radius_vpn_name) > 31:
            module.fail_json(
                msg='Error: The radius_vpn_name %s is large than 31.' % radius_vpn_name)
        if ' ' in radius_vpn_name:
            module.fail_json(
                msg='Error: The radius_vpn_name %s include space.' % radius_vpn_name)

    if radius_server_name:
        if len(radius_server_name) > 255:
            module.fail_json(
                msg='Error: The radius_server_name %s is large than 255.' % radius_server_name)
        if ' ' in radius_server_name:
            module.fail_json(
                msg='Error: The radius_server_name %s include space.' % radius_server_name)

    if hwtacacs_template and len(hwtacacs_template) > 32:
        module.fail_json(
            msg='Error: The hwtacacs_template %s is large than 32.' % hwtacacs_template)

    if hwtacacs_server_ip and not check_ip_addr(hwtacacs_server_ip):
        module.fail_json(
            msg='Error: The hwtacacs_server_ip %s is invalid.' % hwtacacs_server_ip)

    if hwtacacs_vpn_name:
        if len(hwtacacs_vpn_name) > 31:
            module.fail_json(
                msg='Error: The hwtacacs_vpn_name %s is large than 31.' % hwtacacs_vpn_name)
        if ' ' in hwtacacs_vpn_name:
            module.fail_json(
                msg='Error: The hwtacacs_vpn_name %s include space.' % hwtacacs_vpn_name)

    if hwtacacs_server_host_name:
        if len(hwtacacs_server_host_name) > 255:
            module.fail_json(
                msg='Error: The hwtacacs_server_host_name %s is large than 255.' % hwtacacs_server_host_name)
        if ' ' in hwtacacs_server_host_name:
            module.fail_json(
                msg='Error: The hwtacacs_server_host_name %s include space.' % hwtacacs_server_host_name)


def main():
    """ Module main """

    argument_spec = dict(
        state=dict(choices=['present', 'absent'], default='present'),
        local_user_name=dict(type='str'),
        local_password=dict(type='str', no_log=True),
        local_service_type=dict(type='str'),
        local_ftp_dir=dict(type='str'),
        local_user_level=dict(type='str'),
        local_user_group=dict(type='str'),
        radius_group_name=dict(type='str'),
        radius_server_type=dict(choices=['Authentication', 'Accounting']),
        radius_server_ip=dict(type='str'),
        radius_server_ipv6=dict(type='str'),
        radius_server_port=dict(type='str'),
        radius_server_mode=dict(
            choices=['Secondary-server', 'Primary-server']),
        radius_vpn_name=dict(type='str'),
        radius_server_name=dict(type='str'),
        hwtacacs_template=dict(type='str'),
        hwtacacs_server_ip=dict(type='str'),
        hwtacacs_server_ipv6=dict(type='str'),
        hwtacacs_server_type=dict(
            choices=['Authentication', 'Authorization', 'Accounting', 'Common']),
        hwtacacs_is_secondary_server=dict(
            required=False, default=False, type='bool'),
        hwtacacs_vpn_name=dict(type='str'),
        hwtacacs_is_public_net=dict(
            required=False, default=False, type='bool'),
        hwtacacs_server_host_name=dict(type='str')
    )

    argument_spec.update(ce_argument_spec)

    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True)

    check_module_argument(module=module)

    changed = False
    proposed = dict()
    existing = dict()
    end_state = dict()
    updates = []

    # common para
    state = module.params['state']

    # local para
    local_user_name = module.params['local_user_name']
    local_password = module.params['local_password']
    local_service_type = module.params['local_service_type']
    local_ftp_dir = module.params['local_ftp_dir']
    local_user_level = module.params['local_user_level']
    local_user_group = module.params['local_user_group']

    # radius para
    radius_group_name = module.params['radius_group_name']
    radius_server_type = module.params['radius_server_type']
    radius_server_ip = module.params['radius_server_ip']
    radius_server_ipv6 = module.params['radius_server_ipv6']
    radius_server_port = module.params['radius_server_port']
    radius_server_mode = module.params['radius_server_mode']
    radius_vpn_name = module.params['radius_vpn_name']
    radius_server_name = module.params['radius_server_name']

    # hwtacacs para
    hwtacacs_template = module.params['hwtacacs_template']
    hwtacacs_server_ip = module.params['hwtacacs_server_ip']
    hwtacacs_server_ipv6 = module.params['hwtacacs_server_ipv6']
    hwtacacs_server_type = module.params['hwtacacs_server_type']
    hwtacacs_is_secondary_server = module.params[
        'hwtacacs_is_secondary_server']
    hwtacacs_vpn_name = module.params['hwtacacs_vpn_name']
    hwtacacs_is_public_net = module.params['hwtacacs_is_public_net']
    hwtacacs_server_host_name = module.params['hwtacacs_server_host_name']

    ce_aaa_server_host = AaaServerHost()

    if not ce_aaa_server_host:
        module.fail_json(msg='Error: Construct ce_aaa_server failed.')

    # get proposed
    proposed["state"] = state
    if local_user_name:
        proposed["local_user_name"] = local_user_name
    if local_password:
        proposed["local_password"] = "******"
    if local_service_type:
        proposed["local_service_type"] = local_service_type
    if local_ftp_dir:
        proposed["local_ftp_dir"] = local_ftp_dir
    if local_user_level:
        proposed["local_user_level"] = local_user_level
    if local_user_group:
        proposed["local_user_group"] = local_user_group
    if radius_group_name:
        proposed["radius_group_name"] = radius_group_name
    if radius_server_type:
        proposed["radius_server_type"] = radius_server_type
    if radius_server_ip:
        proposed["radius_server_ip"] = radius_server_ip
    if radius_server_ipv6:
        proposed["radius_server_ipv6"] = radius_server_ipv6
    if radius_server_port:
        proposed["radius_server_port"] = radius_server_port
    if radius_server_mode:
        proposed["radius_server_mode"] = radius_server_mode
    if radius_vpn_name:
        proposed["radius_vpn_name"] = radius_vpn_name
    if radius_server_name:
        proposed["radius_server_name"] = radius_server_name
    if hwtacacs_template:
        proposed["hwtacacs_template"] = hwtacacs_template
    if hwtacacs_server_ip:
        proposed["hwtacacs_server_ip"] = hwtacacs_server_ip
    if hwtacacs_server_ipv6:
        proposed["hwtacacs_server_ipv6"] = hwtacacs_server_ipv6
    if hwtacacs_server_type:
        proposed["hwtacacs_server_type"] = hwtacacs_server_type
    proposed["hwtacacs_is_secondary_server"] = hwtacacs_is_secondary_server
    if hwtacacs_vpn_name:
        proposed["hwtacacs_vpn_name"] = hwtacacs_vpn_name
    proposed["hwtacacs_is_public_net"] = hwtacacs_is_public_net
    if hwtacacs_server_host_name:
        proposed["hwtacacs_server_host_name"] = hwtacacs_server_host_name

    if local_user_name:

        if state == "present" and not local_password:
            module.fail_json(
                msg='Error: Please input local_password when config local user.')

        local_user_result = ce_aaa_server_host.get_local_user_info(
            module=module)
        existing["local user name"] = local_user_result["local_user_info"]

        if state == "present":
            # present local user
            if local_user_result["need_cfg"]:
                cmd = ce_aaa_server_host.merge_local_user_info(module=module)

                changed = True
                updates.append(cmd)

        else:
            # absent local user
            if local_user_result["need_cfg"]:
                if not local_service_type and not local_ftp_dir and not local_user_level and not local_user_group:
                    cmd = ce_aaa_server_host.delete_local_user_info(
                        module=module)
                else:
                    cmd = ce_aaa_server_host.merge_local_user_info(
                        module=module)

                changed = True
                updates.append(cmd)

        local_user_result = ce_aaa_server_host.get_local_user_info(
            module=module)
        end_state["local user name"] = local_user_result["local_user_info"]

    if radius_group_name:

        if not radius_server_ip and not radius_server_ipv6 and not radius_server_name:
            module.fail_json(
                msg='Error: Please input radius_server_ip or radius_server_ipv6 or radius_server_name.')

        if radius_server_ip and radius_server_ipv6:
            module.fail_json(
                msg='Error: Please do not input radius_server_ip and radius_server_ipv6 at the same time.')

        if not radius_server_type or not radius_server_port or not radius_server_mode or not radius_vpn_name:
            module.fail_json(
                msg='Error: Please input radius_server_type radius_server_port radius_server_mode radius_vpn_name.')

        if radius_server_ip:
            rds_server_ipv4_result = ce_aaa_server_host.get_radius_server_cfg_ipv4(
                module=module)
        if radius_server_ipv6:
            rds_server_ipv6_result = ce_aaa_server_host.get_radius_server_cfg_ipv6(
                module=module)
        if radius_server_name:
            rds_server_name_result = ce_aaa_server_host.get_radius_server_name(
                module=module)

        if radius_server_ip and rds_server_ipv4_result["radius_server_ip_v4"]:
            existing["radius server ipv4"] = rds_server_ipv4_result[
                "radius_server_ip_v4"]
        if radius_server_ipv6 and rds_server_ipv6_result["radius_server_ip_v6"]:
            existing["radius server ipv6"] = rds_server_ipv6_result[
                "radius_server_ip_v6"]
        if radius_server_name and rds_server_name_result["radius_server_name_cfg"]:
            existing["radius server name cfg"] = rds_server_name_result[
                "radius_server_name_cfg"]

        if state == "present":
            if radius_server_ip and rds_server_ipv4_result["need_cfg"]:
                cmd = ce_aaa_server_host.merge_radius_server_cfg_ipv4(
                    module=module)
                changed = True
                updates.append(cmd)

            if radius_server_ipv6 and rds_server_ipv6_result["need_cfg"]:
                cmd = ce_aaa_server_host.merge_radius_server_cfg_ipv6(
                    module=module)
                changed = True
                updates.append(cmd)

            if radius_server_name and rds_server_name_result["need_cfg"]:
                cmd = ce_aaa_server_host.merge_radius_server_name(
                    module=module)
                changed = True
                updates.append(cmd)
        else:
            if radius_server_ip and rds_server_ipv4_result["need_cfg"]:
                cmd = ce_aaa_server_host.delete_radius_server_cfg_ipv4(
                    module=module)
                changed = True
                updates.append(cmd)

            if radius_server_ipv6 and rds_server_ipv6_result["need_cfg"]:
                cmd = ce_aaa_server_host.delete_radius_server_cfg_ipv6(
                    module=module)
                changed = True
                updates.append(cmd)

            if radius_server_name and rds_server_name_result["need_cfg"]:
                cmd = ce_aaa_server_host.delete_radius_server_name(
                    module=module)
                changed = True
                updates.append(cmd)

        if radius_server_ip:
            rds_server_ipv4_result = ce_aaa_server_host.get_radius_server_cfg_ipv4(
                module=module)
        if radius_server_ipv6:
            rds_server_ipv6_result = ce_aaa_server_host.get_radius_server_cfg_ipv6(
                module=module)
        if radius_server_name:
            rds_server_name_result = ce_aaa_server_host.get_radius_server_name(
                module=module)

        if radius_server_ip and rds_server_ipv4_result["radius_server_ip_v4"]:
            end_state["radius server ipv4"] = rds_server_ipv4_result[
                "radius_server_ip_v4"]
        if radius_server_ipv6 and rds_server_ipv6_result["radius_server_ip_v6"]:
            end_state["radius server ipv6"] = rds_server_ipv6_result[
                "radius_server_ip_v6"]
        if radius_server_name and rds_server_name_result["radius_server_name_cfg"]:
            end_state["radius server name cfg"] = rds_server_name_result[
                "radius_server_name_cfg"]

    if hwtacacs_template:

        if not hwtacacs_server_ip and not hwtacacs_server_ipv6 and not hwtacacs_server_host_name:
            module.fail_json(
                msg='Error: Please input hwtacacs_server_ip or hwtacacs_server_ipv6 or hwtacacs_server_host_name.')

        if not hwtacacs_server_type or not hwtacacs_vpn_name:
            module.fail_json(
                msg='Error: Please input hwtacacs_server_type hwtacacs_vpn_name.')

        if hwtacacs_server_ip and hwtacacs_server_ipv6:
            module.fail_json(
                msg='Error: Please do not set hwtacacs_server_ip and hwtacacs_server_ipv6 at the same time.')

        if hwtacacs_vpn_name and hwtacacs_vpn_name != "_public_":
            if hwtacacs_is_public_net:
                module.fail_json(
                    msg='Error: Please do not set vpn and public net at the same time.')

        if hwtacacs_server_ip:
            hwtacacs_server_ipv4_result = ce_aaa_server_host.get_hwtacacs_server_cfg_ipv4(
                module=module)
        if hwtacacs_server_ipv6:
            hwtacacs_server_ipv6_result = ce_aaa_server_host.get_hwtacacs_server_cfg_ipv6(
                module=module)
        if hwtacacs_server_host_name:
            hwtacacs_host_name_result = ce_aaa_server_host.get_hwtacacs_host_server_cfg(
                module=module)

        if hwtacacs_server_ip and hwtacacs_server_ipv4_result["hwtacacs_server_cfg_ipv4"]:
            existing["hwtacacs server cfg ipv4"] = hwtacacs_server_ipv4_result[
                "hwtacacs_server_cfg_ipv4"]
        if hwtacacs_server_ipv6 and hwtacacs_server_ipv6_result["hwtacacs_server_cfg_ipv6"]:
            existing["hwtacacs server cfg ipv6"] = hwtacacs_server_ipv6_result[
                "hwtacacs_server_cfg_ipv6"]
        if hwtacacs_server_host_name and hwtacacs_host_name_result["hwtacacs_server_name_cfg"]:
            existing["hwtacacs server name cfg"] = hwtacacs_host_name_result[
                "hwtacacs_server_name_cfg"]

        if state == "present":
            if hwtacacs_server_ip and hwtacacs_server_ipv4_result["need_cfg"]:
                cmd = ce_aaa_server_host.merge_hwtacacs_server_cfg_ipv4(
                    module=module)
                changed = True
                updates.append(cmd)

            if hwtacacs_server_ipv6 and hwtacacs_server_ipv6_result["need_cfg"]:
                cmd = ce_aaa_server_host.merge_hwtacacs_server_cfg_ipv6(
                    module=module)
                changed = True
                updates.append(cmd)

            if hwtacacs_server_host_name and hwtacacs_host_name_result["need_cfg"]:
                cmd = ce_aaa_server_host.merge_hwtacacs_host_server_cfg(
                    module=module)
                changed = True
                updates.append(cmd)

        else:
            if hwtacacs_server_ip and hwtacacs_server_ipv4_result["need_cfg"]:
                cmd = ce_aaa_server_host.delete_hwtacacs_server_cfg_ipv4(
                    module=module)
                changed = True
                updates.append(cmd)

            if hwtacacs_server_ipv6 and hwtacacs_server_ipv6_result["need_cfg"]:
                cmd = ce_aaa_server_host.delete_hwtacacs_server_cfg_ipv6(
                    module=module)
                changed = True
                updates.append(cmd)

            if hwtacacs_server_host_name and hwtacacs_host_name_result["need_cfg"]:
                cmd = ce_aaa_server_host.delete_hwtacacs_host_server_cfg(
                    module=module)
                changed = True
                updates.append(cmd)

        if hwtacacs_server_ip:
            hwtacacs_server_ipv4_result = ce_aaa_server_host.get_hwtacacs_server_cfg_ipv4(
                module=module)
        if hwtacacs_server_ipv6:
            hwtacacs_server_ipv6_result = ce_aaa_server_host.get_hwtacacs_server_cfg_ipv6(
                module=module)
        if hwtacacs_server_host_name:
            hwtacacs_host_name_result = ce_aaa_server_host.get_hwtacacs_host_server_cfg(
                module=module)

        if hwtacacs_server_ip and hwtacacs_server_ipv4_result["hwtacacs_server_cfg_ipv4"]:
            end_state["hwtacacs server cfg ipv4"] = hwtacacs_server_ipv4_result[
                "hwtacacs_server_cfg_ipv4"]
        if hwtacacs_server_ipv6 and hwtacacs_server_ipv6_result["hwtacacs_server_cfg_ipv6"]:
            end_state["hwtacacs server cfg ipv6"] = hwtacacs_server_ipv6_result[
                "hwtacacs_server_cfg_ipv6"]
        if hwtacacs_server_host_name and hwtacacs_host_name_result["hwtacacs_server_name_cfg"]:
            end_state["hwtacacs server name cfg"] = hwtacacs_host_name_result[
                "hwtacacs_server_name_cfg"]

    results = dict()
    results['proposed'] = proposed
    results['existing'] = existing
    results['changed'] = changed
    results['end_state'] = end_state
    results['updates'] = updates

    module.exit_json(**results)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = """
module: nxos_bgp
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: (deprecated, removed after 2023-01-27) Manages BGP configuration.
description:
- Manages BGP configurations on NX-OS switches.
version_added: 1.0.0
author:
- Jason Edelman (@jedelman8)
- Gabriele Gerbino (@GGabriele)
deprecated:
  alternative: nxos_bgp_global
  why: Updated module released with more functionality.
  removed_at_date: '2023-01-27'
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- C(state=absent) removes the whole BGP ASN configuration when C(vrf=default) or the
  whole VRF instance within the BGP process when using a different VRF.
- Default when supported restores params default value.
- Configuring global params is only permitted if C(vrf=default).
options:
  asn:
    description:
    - BGP autonomous system number. Valid values are String, Integer in ASPLAIN or
      ASDOT notation.
    required: true
    type: str
  vrf:
    description:
    - Name of the VRF. The name 'default' is a valid VRF representing the global BGP.
    default: 'default'
    type: str
  bestpath_always_compare_med:
    description:
    - Enable/Disable MED comparison on paths from different autonomous systems.
    type: bool
  bestpath_aspath_multipath_relax:
    description:
    - Enable/Disable load sharing across the providers with different (but equal-length)
      AS paths.
    type: bool
  bestpath_compare_routerid:
    description:
    - Enable/Disable comparison of router IDs for identical eBGP paths.
    type: bool
  bestpath_compare_neighborid:
    description:
    - Enable/Disable neighborid. Use this when more paths available than max path
      config.
    type: bool
  bestpath_cost_community_ignore:
    description:
    - Enable/Disable Ignores the cost community for BGP best-path calculations.
    type: bool
  bestpath_med_confed:
    description:
    - Enable/Disable enforcement of bestpath to do a MED comparison only between paths
      originated within a confederation.
    type: bool
  bestpath_med_missing_as_worst:
    description:
    - Enable/Disable assigns the value of infinity to received routes that do not
      carry the MED attribute, making these routes the least desirable.
    type: bool
  bestpath_med_non_deterministic:
    description:
    - Enable/Disable deterministic selection of the best MED pat from among the paths
      from the same autonomous system.
    type: bool
  cluster_id:
    description:
    - Route Reflector Cluster-ID.
    type: str
  confederation_id:
    description:
    - Routing domain confederation AS.
    type: str
  confederation_peers:
    description:
    - AS confederation parameters.
    type: list
    elements: str
  disable_policy_batching:
    description:
    - Enable/Disable the batching evaluation of prefix advertisement to all peers.
    type: bool
  disable_policy_batching_ipv4_prefix_list:
    description:
    - Enable/Disable the batching evaluation of prefix advertisements to all peers
      with prefix list.
    type: str
  disable_policy_batching_ipv6_prefix_list:
    description:
    - Enable/Disable the batching evaluation of prefix advertisements to all peers
      with prefix list.
    type: str
  enforce_first_as:
    description:
    - Enable/Disable enforces the neighbor autonomous system to be the first AS number
      listed in the AS path attribute for eBGP. On NX-OS, this property is only supported
      in the global BGP context.
    type: bool
  event_history_cli:
    description:
    - Enable/Disable cli event history buffer.
    choices:
    - size_small
    - size_medium
    - size_large
    - size_disable
    - default
    - 'true'
    - 'false'
    type: str
  event_history_detail:
    description:
    - Enable/Disable detail event history buffer.
    choices:
    - size_small
    - size_medium
    - size_large
    - size_disable
    - default
    - 'true'
    - 'false'
    type: str
  event_history_events:
    description:
    - Enable/Disable event history buffer.
    choices:
    - size_small
    - size_medium
    - size_large
    - size_disable
    - default
    - 'true'
    - 'false'
    type: str
  event_history_periodic:
    description:
    - Enable/Disable periodic event history buffer.
    choices:
    - size_small
    - size_medium
    - size_large
    - size_disable
    - default
    - 'true'
    - 'false'
    type: str
  fast_external_fallover:
    description:
    - Enable/Disable immediately reset the session if the link to a directly connected
      BGP peer goes down.  Only supported in the global BGP context.
    type: bool
  flush_routes:
    description:
    - Enable/Disable flush routes in RIB upon controlled restart. On NX-OS, this property
      is only supported in the global BGP context.
    type: bool
  graceful_restart:
    description:
    - Enable/Disable graceful restart.
    type: bool
  graceful_restart_helper:
    description:
    - Enable/Disable graceful restart helper mode.
    type: bool
  graceful_restart_timers_restart:
    description:
    - Set maximum time for a restart sent to the BGP peer.
    type: str
  graceful_restart_timers_stalepath_time:
    description:
    - Set maximum time that BGP keeps the stale routes from the restarting BGP peer.
    type: str
  isolate:
    description:
    - Enable/Disable isolate this router from BGP perspective.
    type: bool
  local_as:
    description:
    - Local AS number to be used within a VRF instance.
    type: str
  log_neighbor_changes:
    description:
    - Enable/Disable message logging for neighbor up/down event.
    type: bool
  maxas_limit:
    description:
    - Specify Maximum number of AS numbers allowed in the AS-path attribute. Valid
      values are between 1 and 512.
    type: str
  neighbor_down_fib_accelerate:
    description:
    - Enable/Disable handle BGP neighbor down event, due to various reasons.
    type: bool
  reconnect_interval:
    description:
    - The BGP reconnection interval for dropped sessions. Valid values are between
      1 and 60.
    type: str
  router_id:
    description:
    - Router Identifier (ID) of the BGP router VRF instance.
    type: str
  shutdown:
    description:
    - Administratively shutdown the BGP protocol.
    type: bool
  suppress_fib_pending:
    description:
    - Enable/Disable advertise only routes programmed in hardware to peers.
    type: bool
  timer_bestpath_limit:
    description:
    - Specify timeout for the first best path after a restart, in seconds.
    type: str
  timer_bgp_hold:
    description:
    - Set BGP hold timer.
    type: str
  timer_bgp_keepalive:
    description:
    - Set BGP keepalive timer.
    type: str
  state:
    description:
    - Determines whether the config should be present or not on the device.
    default: present
    choices:
    - present
    - absent
    type: str
"""


EXAMPLES = """
- name: Configure a simple ASN
  cisco.nxos.nxos_bgp:
    asn: 65535
    vrf: test
    router_id: 192.0.2.1
    state: present
"""

RETURN = """
commands:
    description: commands sent to the device
    returned: always
    type: list
    sample: ["router bgp 65535", "vrf test", "router-id 192.0.2.1"]
"""

import re

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    get_config,
    load_config,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    nxos_argument_spec,
)
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.config import (
    CustomNetworkConfig,
)


BOOL_PARAMS = [
    "bestpath_always_compare_med",
    "bestpath_aspath_multipath_relax",
    "bestpath_compare_neighborid",
    "bestpath_compare_routerid",
    "bestpath_cost_community_ignore",
    "bestpath_med_confed",
    "bestpath_med_missing_as_worst",
    "bestpath_med_non_deterministic",
    "disable_policy_batching",
    "enforce_first_as",
    "fast_external_fallover",
    "flush_routes",
    "graceful_restart",
    "graceful_restart_helper",
    "isolate",
    "log_neighbor_changes",
    "neighbor_down_fib_accelerate",
    "shutdown",
    "suppress_fib_pending",
]
GLOBAL_PARAMS = [
    "disable_policy_batching",
    "disable_policy_batching_ipv4_prefix_list",
    "disable_policy_batching_ipv6_prefix_list",
    "enforce_first_as",
    "event_history_cli",
    "event_history_detail",
    "event_history_events",
    "event_history_periodic",
    "fast_external_fallover",
    "flush_routes",
    "isolate",
    "suppress_fib_pending",
    "shutdown",
]
PARAM_TO_DEFAULT_KEYMAP = {
    "timer_bgp_keepalive": "60",
    "timer_bgp_hold": "180",
    "timer_bestpath_limit": "300",
    "graceful_restart": True,
    "graceful_restart_timers_restart": "120",
    "graceful_restart_timers_stalepath_time": "300",
    "reconnect_interval": "60",
    "suppress_fib_pending": True,
    "fast_external_fallover": True,
    "enforce_first_as": True,
    "event_history_cli": True,
    "event_history_detail": False,
    "event_history_events": True,
    "event_history_periodic": True,
    "maxas_limit": "",
    "router_id": "",
    "cluster_id": "",
    "disable_policy_batching_ipv4_prefix_list": "",
    "disable_policy_batching_ipv6_prefix_list": "",
    "local_as": "",
    "confederation_id": "",
}
PARAM_TO_COMMAND_KEYMAP = {
    "asn": "router bgp",
    "bestpath_always_compare_med": "bestpath always-compare-med",
    "bestpath_aspath_multipath_relax": "bestpath as-path multipath-relax",
    "bestpath_compare_neighborid": "bestpath compare-neighborid",
    "bestpath_compare_routerid": "bestpath compare-routerid",
    "bestpath_cost_community_ignore": "bestpath cost-community ignore",
    "bestpath_med_confed": "bestpath med confed",
    "bestpath_med_missing_as_worst": "bestpath med missing-as-worst",
    "bestpath_med_non_deterministic": "bestpath med non-deterministic",
    "cluster_id": "cluster-id",
    "confederation_id": "confederation identifier",
    "confederation_peers": "confederation peers",
    "disable_policy_batching": "disable-policy-batching",
    "disable_policy_batching_ipv4_prefix_list": "disable-policy-batching ipv4 prefix-list",
    "disable_policy_batching_ipv6_prefix_list": "disable-policy-batching ipv6 prefix-list",
    "enforce_first_as": "enforce-first-as",
    "event_history_cli": "event-history cli",
    "event_history_detail": "event-history detail",
    "event_history_events": "event-history events",
    "event_history_periodic": "event-history periodic",
    "fast_external_fallover": "fast-external-fallover",
    "flush_routes": "flush-routes",
    "graceful_restart": "graceful-restart",
    "graceful_restart_helper": "graceful-restart-helper",
    "graceful_restart_timers_restart": "graceful-restart restart-time",
    "graceful_restart_timers_stalepath_time": "graceful-restart stalepath-time",
    "isolate": "isolate",
    "local_as": "local-as",
    "log_neighbor_changes": "log-neighbor-changes",
    "maxas_limit": "maxas-limit",
    "neighbor_down_fib_accelerate": "neighbor-down fib-accelerate",
    "reconnect_interval": "reconnect-interval",
    "router_id": "router-id",
    "shutdown": "shutdown",
    "suppress_fib_pending": "suppress-fib-pending",
    "timer_bestpath_limit": "timers bestpath-limit",
    "timer_bgp_hold": "timers bgp",
    "timer_bgp_keepalive": "timers bgp",
    "vrf": "vrf",
}


def get_value(arg, config):
    command = PARAM_TO_COMMAND_KEYMAP.get(arg)

    if command.split()[0] == "event-history":
        has_size = re.search(
            r"^\s+{0} size\s(?P<value>.*)$".format(command), config, re.M
        )

        if command == "event-history detail":
            value = False
        else:
            value = "size_small"

        if has_size:
            value = "size_%s" % has_size.group("value")

    elif arg in ["enforce_first_as", "fast_external_fallover"]:
        no_command_re = re.compile(r"no\s+{0}\s*".format(command), re.M)
        value = True

        if no_command_re.search(config):
            value = False

    elif arg in BOOL_PARAMS:
        has_command = re.search(r"^\s+{0}\s*$".format(command), config, re.M)
        value = False

        if has_command:
            value = True
    else:
        command_val_re = re.compile(
            r"(?:{0}\s)(?P<value>.*)".format(command), re.M
        )
        value = ""

        has_command = command_val_re.search(config)
        if has_command:
            found_value = has_command.group("value")

            if arg == "confederation_peers":
                value = found_value.split()
            elif arg == "timer_bgp_keepalive":
                value = found_value.split()[0]
            elif arg == "timer_bgp_hold":
                split_values = found_value.split()
                if len(split_values) == 2:
                    value = split_values[1]
            elif found_value:
                value = found_value

    return value


def get_existing(module, args, warnings):
    existing = {}
    netcfg = CustomNetworkConfig(
        indent=2, contents=get_config(module, flags=["bgp all"])
    )

    asn_re = re.compile(
        r".*router\sbgp\s(?P<existing_asn>\d+(\.\d+)?).*", re.S
    )
    asn_match = asn_re.match(str(netcfg))

    if asn_match:
        existing_asn = asn_match.group("existing_asn")
        bgp_parent = "router bgp {0}".format(existing_asn)

        if module.params["vrf"] != "default":
            parents = [bgp_parent, "vrf {0}".format(module.params["vrf"])]
        else:
            parents = [bgp_parent]

        config = netcfg.get_section(parents)
        if config:
            for arg in args:
                if arg != "asn" and (
                    module.params["vrf"] == "default"
                    or arg not in GLOBAL_PARAMS
                ):
                    existing[arg] = get_value(arg, config)

            existing["asn"] = existing_asn
            if module.params["vrf"] == "default":
                existing["vrf"] = "default"

    if (
        not existing
        and module.params["vrf"] != "default"
        and module.params["state"] == "present"
    ):
        msg = "VRF {0} doesn't exist.".format(module.params["vrf"])
        warnings.append(msg)

    return existing


def apply_key_map(key_map, table):
    new_dict = {}
    for key in table:
        new_key = key_map.get(key)
        if new_key:
            new_dict[new_key] = table.get(key)

    return new_dict


def state_present(module, existing, proposed, candidate):
    commands = list()
    proposed_commands = apply_key_map(PARAM_TO_COMMAND_KEYMAP, proposed)
    existing_commands = apply_key_map(PARAM_TO_COMMAND_KEYMAP, existing)

    for key, value in proposed_commands.items():
        if value is True:
            commands.append(key)
        elif value is False:
            commands.append("no {0}".format(key))
        elif value == "default":
            default_value = PARAM_TO_DEFAULT_KEYMAP.get(key)
            existing_value = existing_commands.get(key)

            if default_value:
                commands.append("{0} {1}".format(key, default_value))
            elif existing_value:
                if key == "confederation peers":
                    existing_value = " ".join(existing_value)
                commands.append("no {0} {1}".format(key, existing_value))
        elif not value:
            existing_value = existing_commands.get(key)
            if existing_value:
                commands.append("no {0} {1}".format(key, existing_value))
        elif key == "confederation peers":
            commands.append("{0} {1}".format(key, value))
        elif key.startswith("timers bgp"):
            command = "timers bgp {0} {1}".format(
                proposed["timer_bgp_keepalive"], proposed["timer_bgp_hold"]
            )
            if command not in commands:
                commands.append(command)
        else:
            if value.startswith("size"):
                value = value.replace("_", " ")
            command = "{0} {1}".format(key, value)
            commands.append(command)

    parents = []
    if commands:
        commands = fix_commands(commands)
        parents = ["router bgp {0}".format(module.params["asn"])]
        if module.params["vrf"] != "default":
            parents.append("vrf {0}".format(module.params["vrf"]))
    elif proposed:
        if module.params["vrf"] != "default":
            commands.append("vrf {0}".format(module.params["vrf"]))
            parents = ["router bgp {0}".format(module.params["asn"])]
        else:
            commands.append("router bgp {0}".format(module.params["asn"]))

    candidate.add(commands, parents=parents)


def state_absent(module, existing, candidate):
    commands = []
    parents = []
    if module.params["vrf"] == "default":
        commands.append("no router bgp {0}".format(module.params["asn"]))
    elif existing.get("vrf") == module.params["vrf"]:
        commands.append("no vrf {0}".format(module.params["vrf"]))
        parents = ["router bgp {0}".format(module.params["asn"])]

    candidate.add(commands, parents=parents)


def fix_commands(commands):
    local_as_command = ""
    confederation_id_command = ""
    confederation_peers_command = ""

    for command in commands:
        if "local-as" in command:
            local_as_command = command
        elif "confederation identifier" in command:
            confederation_id_command = command
        elif "confederation peers" in command:
            confederation_peers_command = command

    if local_as_command and confederation_id_command:
        if "no" in confederation_id_command:
            commands.pop(commands.index(local_as_command))
            commands.pop(commands.index(confederation_id_command))
            commands.append(confederation_id_command)
            commands.append(local_as_command)
        else:
            commands.pop(commands.index(local_as_command))
            commands.pop(commands.index(confederation_id_command))
            commands.append(local_as_command)
            commands.append(confederation_id_command)

    if confederation_peers_command and confederation_id_command:
        if local_as_command:
            if "no" in local_as_command:
                commands.pop(commands.index(local_as_command))
                commands.pop(commands.index(confederation_id_command))
                commands.pop(commands.index(confederation_peers_command))
                commands.append(confederation_id_command)
                commands.append(confederation_peers_command)
                commands.append(local_as_command)
            else:
                commands.pop(commands.index(local_as_command))
                commands.pop(commands.index(confederation_id_command))
                commands.pop(commands.index(confederation_peers_command))
                commands.append(local_as_command)
                commands.append(confederation_id_command)
                commands.append(confederation_peers_command)
        else:
            commands.pop(commands.index(confederation_peers_command))
            commands.pop(commands.index(confederation_id_command))
            commands.append(confederation_id_command)
            commands.append(confederation_peers_command)

    return commands


def main():
    argument_spec = dict(
        asn=dict(required=True, type="str"),
        vrf=dict(required=False, type="str", default="default"),
        bestpath_always_compare_med=dict(required=False, type="bool"),
        bestpath_aspath_multipath_relax=dict(required=False, type="bool"),
        bestpath_compare_neighborid=dict(required=False, type="bool"),
        bestpath_compare_routerid=dict(required=False, type="bool"),
        bestpath_cost_community_ignore=dict(required=False, type="bool"),
        bestpath_med_confed=dict(required=False, type="bool"),
        bestpath_med_missing_as_worst=dict(required=False, type="bool"),
        bestpath_med_non_deterministic=dict(required=False, type="bool"),
        cluster_id=dict(required=False, type="str"),
        confederation_id=dict(required=False, type="str"),
        confederation_peers=dict(required=False, type="list", elements="str"),
        disable_policy_batching=dict(required=False, type="bool"),
        disable_policy_batching_ipv4_prefix_list=dict(
            required=False, type="str"
        ),
        disable_policy_batching_ipv6_prefix_list=dict(
            required=False, type="str"
        ),
        enforce_first_as=dict(required=False, type="bool"),
        event_history_cli=dict(
            required=False,
            choices=[
                "true",
                "false",
                "default",
                "size_small",
                "size_medium",
                "size_large",
                "size_disable",
            ],
        ),
        event_history_detail=dict(
            required=False,
            choices=[
                "true",
                "false",
                "default",
                "size_small",
                "size_medium",
                "size_large",
                "size_disable",
            ],
        ),
        event_history_events=dict(
            required=False,
            choices=[
                "true",
                "false",
                "default",
                "size_small",
                "size_medium",
                "size_large",
                "size_disable",
            ],
        ),
        event_history_periodic=dict(
            required=False,
            choices=[
                "true",
                "false",
                "default",
                "size_small",
                "size_medium",
                "size_large",
                "size_disable",
            ],
        ),
        fast_external_fallover=dict(required=False, type="bool"),
        flush_routes=dict(required=False, type="bool"),
        graceful_restart=dict(required=False, type="bool"),
        graceful_restart_helper=dict(required=False, type="bool"),
        graceful_restart_timers_restart=dict(required=False, type="str"),
        graceful_restart_timers_stalepath_time=dict(
            required=False, type="str"
        ),
        isolate=dict(required=False, type="bool"),
        local_as=dict(required=False, type="str"),
        log_neighbor_changes=dict(required=False, type="bool"),
        maxas_limit=dict(required=False, type="str"),
        neighbor_down_fib_accelerate=dict(required=False, type="bool"),
        reconnect_interval=dict(required=False, type="str"),
        router_id=dict(required=False, type="str"),
        shutdown=dict(required=False, type="bool"),
        suppress_fib_pending=dict(required=False, type="bool"),
        timer_bestpath_limit=dict(required=False, type="str"),
        timer_bgp_hold=dict(required=False, type="str"),
        timer_bgp_keepalive=dict(required=False, type="str"),
        state=dict(
            choices=["present", "absent"], default="present", required=False
        ),
    )
    argument_spec.update(nxos_argument_spec)

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_together=[["timer_bgp_hold", "timer_bgp_keepalive"]],
        supports_check_mode=True,
    )

    warnings = list()
    result = dict(changed=False, warnings=warnings)

    state = module.params["state"]

    if module.params["vrf"] != "default":
        for param in GLOBAL_PARAMS:
            if module.params[param]:
                module.fail_json(
                    msg='Global params can be modified only under "default" VRF.',
                    vrf=module.params["vrf"],
                    global_param=param,
                )

    args = PARAM_TO_COMMAND_KEYMAP.keys()
    existing = get_existing(module, args, warnings)

    if existing.get("asn") and state == "present":
        if existing.get("asn") != module.params["asn"]:
            module.fail_json(
                msg="Another BGP ASN already exists.",
                proposed_asn=module.params["asn"],
                existing_asn=existing.get("asn"),
            )

    proposed_args = dict(
        (k, v) for k, v in module.params.items() if v is not None and k in args
    )
    proposed = {}
    for key, value in proposed_args.items():
        if key not in ["asn", "vrf"]:
            if str(value).lower() == "default":
                value = PARAM_TO_DEFAULT_KEYMAP.get(key, "default")
            if key == "confederation_peers":
                if value[0] == "default":
                    if existing.get(key):
                        proposed[key] = "default"
                else:
                    v = set([int(i) for i in value])
                    ex = set([int(i) for i in existing.get(key)])
                    if v != ex:
                        proposed[key] = " ".join(str(s) for s in v)
            else:
                if existing.get(key) != value:
                    proposed[key] = value

    candidate = CustomNetworkConfig(indent=3)
    if state == "present":
        state_present(module, existing, proposed, candidate)
    elif existing.get("asn") == module.params["asn"]:
        state_absent(module, existing, candidate)

    if candidate:
        candidate = candidate.items_text()
        if not module.check_mode:
            load_config(module, candidate)
        result["changed"] = True
        result["commands"] = candidate
    else:
        result["commands"] = []

    module.exit_json(**result)


if __name__ == "__main__":
    main()



from __future__ import absolute_import, division, print_function

__metaclass__ = type
import json

from ansible_collections.cisco.nxos.tests.unit.compat import unittest
from ansible_collections.cisco.nxos.tests.unit.compat.mock import patch
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes


def set_module_args(args):
    if "_ansible_remote_tmp" not in args:
        args["_ansible_remote_tmp"] = "/tmp"
    if "_ansible_keep_remote_files" not in args:
        args["_ansible_keep_remote_files"] = False

    args = json.dumps({"ANSIBLE_MODULE_ARGS": args})
    basic._ANSIBLE_ARGS = to_bytes(args)


class AnsibleExitJson(Exception):
    pass


class AnsibleFailJson(Exception):
    pass


def exit_json(*args, **kwargs):
    if "changed" not in kwargs:
        kwargs["changed"] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):
    kwargs["failed"] = True
    raise AnsibleFailJson(kwargs)


class ModuleTestCase(unittest.TestCase):
    def setUp(self):
        self.mock_module = patch.multiple(
            basic.AnsibleModule, exit_json=exit_json, fail_json=fail_json
        )
        self.mock_module.start()
        self.mock_sleep = patch("time.sleep")
        self.mock_sleep.start()
        set_module_args({})
        self.addCleanup(self.mock_module.stop)
        self.addCleanup(self.mock_sleep.stop)



#!/usr/bin/python
# Copyright 2017 Google Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: gcp_healthcheck
short_description: Create, Update or Destroy a Healthcheck.
description:
    - Create, Update or Destroy a Healthcheck. Currently only HTTP and
      HTTPS Healthchecks are supported. Healthchecks are used to monitor
      individual instances, managed instance groups and/or backend
      services. Healtchecks are reusable.
    - Visit
      U(https://cloud.google.com/compute/docs/load-balancing/health-checks)
      for an overview of Healthchecks on GCP.
    - See
      U(https://cloud.google.com/compute/docs/reference/latest/httpHealthChecks) for
      API details on HTTP Healthchecks.
    - See
      U(https://cloud.google.com/compute/docs/reference/latest/httpsHealthChecks)
      for more details on the HTTPS Healtcheck API.
requirements:
  - "python >= 2.6"
  - "google-api-python-client >= 1.6.2"
  - "google-auth >= 0.9.0"
  - "google-auth-httplib2 >= 0.0.2"
notes:
  - Only supports HTTP and HTTPS Healthchecks currently.
deprecated:
    removed_in: 2.0.0  # was Ansible 2.12
    why: Updated modules released with increased functionality
    alternative: >
      Use M(google.cloud.gcp_compute_health_check), M(google.cloud.gcp_compute_http_health_check) or
      M(google.cloud.gcp_compute_https_health_check) instead.
author:
  - "Tom Melendez (@supertom) <tom@supertom.com>"
options:
  check_interval:
    type: int
    description:
       - How often (in seconds) to send a health check.
    default: 5
  healthcheck_name:
    type: str
    description:
       - Name of the Healthcheck.
    required: true
  healthcheck_type:
    type: str
    description:
       - Type of Healthcheck.
    required: true
    choices: ["HTTP", "HTTPS"]
  host_header:
    type: str
    description:
       - The value of the host header in the health check request. If left
         empty, the public IP on behalf of which this health
         check is performed will be used.
    default: ""
  port:
    type: int
    description:
       - The TCP port number for the health check request. The default value is
         443 for HTTPS and 80 for HTTP.
  request_path:
    type: str
    description:
       - The request path of the HTTPS health check request.
    required: false
    default: "/"
  state:
    type: str
    description: State of the Healthcheck.
    choices: ["present", "absent"]
    default: present
  timeout:
    type: int
    description:
       - How long (in seconds) to wait for a response before claiming
         failure. It is invalid for timeout
         to have a greater value than check_interval.
    default: 5
  unhealthy_threshold:
    type: int
    description:
       - A so-far healthy instance will be marked unhealthy after this
         many consecutive failures.
    default: 2
  healthy_threshold:
    type: int
    description:
       - A so-far unhealthy instance will be marked healthy after this
         many consecutive successes.
    default: 2
  service_account_email:
    type: str
    description:
      - service account email
  service_account_permissions:
    type: list
    description:
      - service account permissions (see
        U(https://cloud.google.com/sdk/gcloud/reference/compute/instances/create),
        --scopes section for detailed information)
      - >
        Available choices are:
        C(bigquery), C(cloud-platform), C(compute-ro), C(compute-rw),
        C(useraccounts-ro), C(useraccounts-rw), C(datastore), C(logging-write),
        C(monitoring), C(sql-admin), C(storage-full), C(storage-ro),
        C(storage-rw), C(taskqueue), C(userinfo-email).
  credentials_file:
    type: str
    description:
      - Path to the JSON file associated with the service account email
  project_id:
    type: str
    description:
      - Your GCP project ID
'''

EXAMPLES = '''
- name: Create Minimum HealthCheck
  community.general.gcp_healthcheck:
    service_account_email: "{{ service_account_email }}"
    credentials_file: "{{ credentials_file }}"
    project_id: "{{ project_id }}"
    healthcheck_name: my-healthcheck
    healthcheck_type: HTTP
    state: present
- name: Create HTTP HealthCheck
  community.general.gcp_healthcheck:
    service_account_email: "{{ service_account_email }}"
    credentials_file: "{{ credentials_file }}"
    project_id: "{{ project_id }}"
    healthcheck_name: my-healthcheck
    healthcheck_type: HTTP
    host: my-host
    request_path: /hc
    check_interval: 10
    timeout: 30
    unhealthy_threshhold: 2
    healthy_threshhold: 1
    state: present
- name: Create HTTPS HealthCheck
  community.general.gcp_healthcheck:
    service_account_email: "{{ service_account_email }}"
    credentials_file: "{{ credentials_file }}"
    project_id: "{{ project_id }}"
    healthcheck_name: "{{ https_healthcheck }}"
    healthcheck_type: HTTPS
    host_header: my-host
    request_path: /hc
    check_interval: 5
    timeout: 5
    unhealthy_threshold: 2
    healthy_threshold: 1
    state: present
'''

RETURN = '''
state:
    description: state of the Healthcheck
    returned: Always.
    type: str
    sample: present
healthcheck_name:
    description: Name of the Healthcheck
    returned: Always
    type: str
    sample: my-url-map
healthcheck_type:
    description: Type of the Healthcheck
    returned: Always
    type: str
    sample: HTTP
healthcheck:
    description: GCP Healthcheck dictionary
    returned: Always. Refer to GCP documentation for detailed field descriptions.
    type: dict
    sample: { "name": "my-hc", "port": 443, "requestPath": "/foo" }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.general.plugins.module_utils.gcp import get_google_api_client, GCPUtils


USER_AGENT_PRODUCT = 'ansible-healthcheck'
USER_AGENT_VERSION = '0.0.1'


def _validate_healthcheck_params(params):
    """
    Validate healthcheck params.

    Simple validation has already assumed by AnsibleModule.

    :param params: Ansible dictionary containing configuration.
    :type  params: ``dict``

    :return: True or raises ValueError
    :rtype: ``bool`` or `class:ValueError`
    """
    if params['timeout'] > params['check_interval']:
        raise ValueError("timeout (%s) is greater than check_interval (%s)" % (
            params['timeout'], params['check_interval']))

    return (True, '')


def _build_healthcheck_dict(params):
    """
    Reformat services in Ansible Params for GCP.

    :param params: Params from AnsibleModule object
    :type params: ``dict``

    :param project_id: The GCP project ID.
    :type project_id:  ``str``

    :return: dictionary suitable for submission to GCP
             HealthCheck (HTTP/HTTPS) API.
    :rtype ``dict``
    """
    gcp_dict = GCPUtils.params_to_gcp_dict(params, 'healthcheck_name')
    if 'timeout' in gcp_dict:
        gcp_dict['timeoutSec'] = gcp_dict['timeout']
        del gcp_dict['timeout']

    if 'checkInterval' in gcp_dict:
        gcp_dict['checkIntervalSec'] = gcp_dict['checkInterval']
        del gcp_dict['checkInterval']

    if 'hostHeader' in gcp_dict:
        gcp_dict['host'] = gcp_dict['hostHeader']
        del gcp_dict['hostHeader']

    if 'healthcheckType' in gcp_dict:
        del gcp_dict['healthcheckType']
    return gcp_dict


def _get_req_resource(client, resource_type):
    if resource_type == 'HTTPS':
        return (client.httpsHealthChecks(), 'httpsHealthCheck')
    else:
        return (client.httpHealthChecks(), 'httpHealthCheck')


def get_healthcheck(client, name, project_id=None, resource_type='HTTP'):
    """
    Get a Healthcheck from GCP.

    :param client: An initialized GCE Compute Discovery resource.
    :type client:  :class: `googleapiclient.discovery.Resource`

    :param name: Name of the Url Map.
    :type name:  ``str``

    :param project_id: The GCP project ID.
    :type project_id:  ``str``

    :return: A dict resp from the respective GCP 'get' request.
    :rtype: ``dict``
    """
    try:
        resource, entity_name = _get_req_resource(client, resource_type)
        args = {'project': project_id, entity_name: name}
        req = resource.get(**args)
        return GCPUtils.execute_api_client_req(req, raise_404=False)
    except Exception:
        raise


def create_healthcheck(client, params, project_id, resource_type='HTTP'):
    """
    Create a new Healthcheck.

    :param client: An initialized GCE Compute Discovery resource.
    :type client:  :class: `googleapiclient.discovery.Resource`

    :param params: Dictionary of arguments from AnsibleModule.
    :type params:  ``dict``

    :return: Tuple with changed status and response dict
    :rtype: ``tuple`` in the format of (bool, dict)
    """
    gcp_dict = _build_healthcheck_dict(params)
    try:
        resource, _ = _get_req_resource(client, resource_type)
        args = {'project': project_id, 'body': gcp_dict}
        req = resource.insert(**args)
        return_data = GCPUtils.execute_api_client_req(req, client, raw=False)
        if not return_data:
            return_data = get_healthcheck(client,
                                          name=params['healthcheck_name'],
                                          project_id=project_id)
        return (True, return_data)
    except Exception:
        raise


def delete_healthcheck(client, name, project_id, resource_type='HTTP'):
    """
    Delete a Healthcheck.

    :param client: An initialized GCE Compute Discovery resource.
    :type client:  :class: `googleapiclient.discovery.Resource`

    :param name: Name of the Url Map.
    :type name:  ``str``

    :param project_id: The GCP project ID.
    :type project_id:  ``str``

    :return: Tuple with changed status and response dict
    :rtype: ``tuple`` in the format of (bool, dict)
    """
    try:
        resource, entity_name = _get_req_resource(client, resource_type)
        args = {'project': project_id, entity_name: name}
        req = resource.delete(**args)
        return_data = GCPUtils.execute_api_client_req(req, client)
        return (True, return_data)
    except Exception:
        raise


def update_healthcheck(client, healthcheck, params, name, project_id,
                       resource_type='HTTP'):
    """
    Update a Healthcheck.

    If the healthcheck has not changed, the update will not occur.

    :param client: An initialized GCE Compute Discovery resource.
    :type client:  :class: `googleapiclient.discovery.Resource`

    :param healthcheck: Name of the Url Map.
    :type healthcheck:  ``dict``

    :param params: Dictionary of arguments from AnsibleModule.
    :type params:  ``dict``

    :param name: Name of the Url Map.
    :type name:  ``str``

    :param project_id: The GCP project ID.
    :type project_id:  ``str``

    :return: Tuple with changed status and response dict
    :rtype: ``tuple`` in the format of (bool, dict)
    """
    gcp_dict = _build_healthcheck_dict(params)
    ans = GCPUtils.are_params_equal(healthcheck, gcp_dict)
    if ans:
        return (False, 'no update necessary')

    try:
        resource, entity_name = _get_req_resource(client, resource_type)
        args = {'project': project_id, entity_name: name, 'body': gcp_dict}
        req = resource.update(**args)
        return_data = GCPUtils.execute_api_client_req(
            req, client=client, raw=False)
        return (True, return_data)
    except Exception:
        raise


def main():
    module = AnsibleModule(argument_spec=dict(
        healthcheck_name=dict(required=True),
        healthcheck_type=dict(required=True,
                              choices=['HTTP', 'HTTPS']),
        request_path=dict(required=False, default='/'),
        check_interval=dict(required=False, type='int', default=5),
        healthy_threshold=dict(required=False, type='int', default=2),
        unhealthy_threshold=dict(required=False, type='int', default=2),
        host_header=dict(required=False, type='str', default=''),
        timeout=dict(required=False, type='int', default=5),
        port=dict(required=False, type='int'),
        state=dict(choices=['absent', 'present'], default='present'),
        service_account_email=dict(),
        service_account_permissions=dict(type='list'),
        credentials_file=dict(),
        project_id=dict(), ), )

    client, conn_params = get_google_api_client(module, 'compute', user_agent_product=USER_AGENT_PRODUCT,
                                                user_agent_version=USER_AGENT_VERSION)

    params = {}

    params['healthcheck_name'] = module.params.get('healthcheck_name')
    params['healthcheck_type'] = module.params.get('healthcheck_type')
    params['request_path'] = module.params.get('request_path')
    params['check_interval'] = module.params.get('check_interval')
    params['healthy_threshold'] = module.params.get('healthy_threshold')
    params['unhealthy_threshold'] = module.params.get('unhealthy_threshold')
    params['host_header'] = module.params.get('host_header')
    params['timeout'] = module.params.get('timeout')
    params['port'] = module.params.get('port', None)
    params['state'] = module.params.get('state')

    if not params['port']:
        params['port'] = 80
        if params['healthcheck_type'] == 'HTTPS':
            params['port'] = 443
    try:
        _validate_healthcheck_params(params)
    except Exception as e:
        module.fail_json(msg=e.message, changed=False)

    changed = False
    json_output = {'state': params['state']}
    healthcheck = get_healthcheck(client,
                                  name=params['healthcheck_name'],
                                  project_id=conn_params['project_id'],
                                  resource_type=params['healthcheck_type'])

    if not healthcheck:
        if params['state'] == 'absent':
            # Doesn't exist in GCE, and state==absent.
            changed = False
            module.fail_json(
                msg="Cannot delete unknown healthcheck: %s" %
                (params['healthcheck_name']))
        else:
            # Create
            changed, json_output['healthcheck'] = create_healthcheck(client,
                                                                     params=params,
                                                                     project_id=conn_params['project_id'],
                                                                     resource_type=params['healthcheck_type'])
    elif params['state'] == 'absent':
        # Delete
        changed, json_output['healthcheck'] = delete_healthcheck(client,
                                                                 name=params['healthcheck_name'],
                                                                 project_id=conn_params['project_id'],
                                                                 resource_type=params['healthcheck_type'])
    else:
        changed, json_output['healthcheck'] = update_healthcheck(client,
                                                                 healthcheck=healthcheck,
                                                                 params=params,
                                                                 name=params['healthcheck_name'],
                                                                 project_id=conn_params['project_id'],
                                                                 resource_type=params['healthcheck_type'])
    json_output['changed'] = changed
    json_output.update(params)
    module.exit_json(**json_output)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright (c) 2016-2017 Hewlett Packard Enterprise Development LP
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = '''
---
module: oneview_network_set_info
short_description: Retrieve information about the OneView Network Sets
description:
    - Retrieve information about the Network Sets from OneView.
    - This module was called C(oneview_network_set_facts) before Ansible 2.9, returning C(ansible_facts).
      Note that the M(community.general.oneview_network_set_info) module no longer returns C(ansible_facts)!
requirements:
    - hpOneView >= 2.0.1
author:
    - Felipe Bulsoni (@fgbulsoni)
    - Thiago Miotto (@tmiotto)
    - Adriane Cardozo (@adriane-cardozo)
options:
    name:
      description:
        - Network Set name.

    options:
      description:
        - "List with options to gather information about Network Set.
          Option allowed: C(withoutEthernet).
          The option C(withoutEthernet) retrieves the list of network_sets excluding Ethernet networks."

extends_documentation_fragment:
- community.general.oneview
- community.general.oneview.factsparams

'''

EXAMPLES = '''
- name: Gather information about all Network Sets
  community.general.oneview_network_set_info:
    hostname: 172.16.101.48
    username: administrator
    password: my_password
    api_version: 500
  no_log: true
  delegate_to: localhost
  register: result

- ansible.builtin.debug:
    msg: "{{ result.network_sets }}"

- name: Gather paginated, filtered, and sorted information about Network Sets
  community.general.oneview_network_set_info:
    hostname: 172.16.101.48
    username: administrator
    password: my_password
    api_version: 500
    params:
      start: 0
      count: 3
      sort: 'name:descending'
      filter: name='netset001'
  no_log: true
  delegate_to: localhost
  register: result

- ansible.builtin.debug:
    msg: "{{ result.network_sets }}"

- name: Gather information about all Network Sets, excluding Ethernet networks
  community.general.oneview_network_set_info:
    hostname: 172.16.101.48
    username: administrator
    password: my_password
    api_version: 500
    options:
        - withoutEthernet
  no_log: true
  delegate_to: localhost
  register: result

- ansible.builtin.debug:
    msg: "{{ result.network_sets }}"

- name: Gather information about a Network Set by name
  community.general.oneview_network_set_info:
    hostname: 172.16.101.48
    username: administrator
    password: my_password
    api_version: 500
    name: Name of the Network Set
  no_log: true
  delegate_to: localhost
  register: result

- ansible.builtin.debug:
    msg: "{{ result.network_sets }}"

- name: Gather information about a Network Set by name, excluding Ethernet networks
  community.general.oneview_network_set_info:
    hostname: 172.16.101.48
    username: administrator
    password: my_password
    api_version: 500
    name: Name of the Network Set
    options:
        - withoutEthernet
  no_log: true
  delegate_to: localhost
  register: result

- ansible.builtin.debug:
    msg: "{{ result.network_sets }}"
'''

RETURN = '''
network_sets:
    description: Has all the OneView information about the Network Sets.
    returned: Always, but can be empty.
    type: dict
'''

from ansible_collections.community.general.plugins.module_utils.oneview import OneViewModuleBase


class NetworkSetInfoModule(OneViewModuleBase):
    argument_spec = dict(
        name=dict(type='str'),
        options=dict(type='list'),
        params=dict(type='dict'),
    )

    def __init__(self):
        super(NetworkSetInfoModule, self).__init__(additional_arg_spec=self.argument_spec)
        self.is_old_facts = self.module._name in ('oneview_network_set_facts', 'community.general.oneview_network_set_facts')
        if self.is_old_facts:
            self.module.deprecate("The 'oneview_network_set_facts' module has been renamed to 'oneview_network_set_info', "
                                  "and the renamed one no longer returns ansible_facts",
                                  version='3.0.0', collection_name='community.general')  # was Ansible 2.13

    def execute_module(self):

        name = self.module.params.get('name')

        if 'withoutEthernet' in self.options:
            filter_by_name = ("\"'name'='%s'\"" % name) if name else ''
            network_sets = self.oneview_client.network_sets.get_all_without_ethernet(filter=filter_by_name)
        elif name:
            network_sets = self.oneview_client.network_sets.get_by('name', name)
        else:
            network_sets = self.oneview_client.network_sets.get_all(**self.facts_params)

        if self.is_old_facts:
            return dict(changed=False,
                        ansible_facts=dict(network_sets=network_sets))
        else:
            return dict(changed=False, network_sets=network_sets)


def main():
    NetworkSetInfoModule().run()


if __name__ == '__main__':
    main()
